@@ -17,18 +17,16 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <string.h>
-#include <config.h>
-#include <gnome.h>
-#include <applet-widget.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <libgnome/libgnome.h>
+#include <libgnomeui/libgnomeui.h>
+#include <panel-applet.h>
+#include <panel-applet-gconf.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-
+#include <libgnomeui/gnome-window-icon.h>
 #include "popcheck.h"
 #include "remote-helper.h"
 #include "mailcheck.h"
 
-GtkWidget *applet = NULL;
-
 typedef enum {
 	MAILBOX_LOCAL,
 	MAILBOX_LOCALDIR,
@@ -67,6 +65,7 @@ struct _MailCheck {
 	char *pre_check_cmd;
 	gboolean pre_check_enabled;	
 
+	PanelApplet *applet;
 	/* This is the event box for catching events */
 	GtkWidget *ebox;
 
@@ -146,6 +145,9 @@ struct _MailCheck {
 static int mail_check_timeout (gpointer data);
 static void after_mail_check (MailCheck *mc);
 
+static void applet_load_prefs(MailCheck *mc);
+static void applet_save_prefs(MailCheck *mc);
+
 #define WANT_BITMAPS(x) (x == REPORT_MAIL_USE_ANIMATION || x == REPORT_MAIL_USE_BITMAP)
 
 static void close_callback (GtkWidget *widget, void *data);
@@ -173,7 +175,8 @@ mail_animation_filename (MailCheck *mc)
 }
 
 static int
-calc_dir_contents (char *dir) {
+calc_dir_contents (char *dir)
+{
        DIR *dr;
        struct dirent *de;
        int size=0;
@@ -361,7 +364,7 @@ mailcheck_load_animation (MailCheck *mc, const char *fname)
 	mc->email_pixmap = NULL;
 	mc->email_mask = NULL;
 
-	pb = gdk_pixbuf_new_from_file (fname);
+	pb = gdk_pixbuf_new_from_file (fname, NULL);
 	if (pb == NULL)
 		return FALSE;
 
@@ -533,14 +536,16 @@ icon_expose (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 }
 
 static gint
-exec_clicked_cmd (GtkWidget *widget, GdkEvent *evt, gpointer data)
+exec_clicked_cmd (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-         MailCheck *mc = data;
-	 if (mc->clicked_enabled && 
-	     mc->clicked_cmd && 
-	     (strlen(mc->clicked_cmd) > 0) )
-	        gnome_execute_shell(NULL, mc->clicked_cmd);
-	 return TRUE;
+	MailCheck *mc = data;
+	gboolean retval = FALSE;
+
+	if (event->button < 3 && mc->clicked_enabled && mc->clicked_cmd && (strlen(mc->clicked_cmd) > 0)) {
+		gnome_execute_shell(NULL, mc->clicked_cmd);
+		retval = TRUE;
+	}
+	return(retval);
 }
 
 static void
@@ -607,7 +612,7 @@ create_mail_widgets (MailCheck *mc)
 	 * applet is removed from the panel while the dialog is
 	 * active.
 	 */
-	gtk_signal_connect (GTK_OBJECT (mc->ebox), "destroy",
+	g_signal_connect (G_OBJECT (mc->ebox), "destroy",
 			    (GtkSignalFunc) mailcheck_destroy,
 			    mc);
 
@@ -619,15 +624,11 @@ create_mail_widgets (MailCheck *mc)
 	mc->mail_timeout = gtk_timeout_add (mc->update_freq, mail_check_timeout, mc);
 
 	/* The drawing area */
-	gtk_widget_push_visual (gdk_imlib_get_visual ());
-	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	mc->da = gtk_drawing_area_new ();
-	gtk_widget_pop_colormap ();
-	gtk_widget_pop_visual ();
 	gtk_widget_ref (mc->da);
 	gtk_drawing_area_size (GTK_DRAWING_AREA(mc->da),
 			       mc->size, mc->size);
-	gtk_signal_connect (GTK_OBJECT(mc->da), "expose_event", (GtkSignalFunc)icon_expose, mc);
+	g_signal_connect (G_OBJECT(mc->da), "expose_event", (GtkSignalFunc)icon_expose, mc);
 	gtk_widget_show (mc->da);
 
 	/* The label */
@@ -651,7 +652,7 @@ create_mail_widgets (MailCheck *mc)
 static void
 set_selection (GtkWidget *widget, gpointer data)
 {
-	MailCheck *mc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	MailCheck *mc = g_object_get_data(G_OBJECT(widget), "MailCheck");
 	mc->selected_pixmap_name = data;
 	mc->anim_changed = TRUE;
 
@@ -671,7 +672,7 @@ mailcheck_new_entry (MailCheck *mc, GtkWidget *menu, GtkWidget *item, char *s)
 {
 	gtk_menu_append (GTK_MENU (menu), item);
 
-	gtk_object_set_user_data (GTK_OBJECT (item), mc);
+	g_object_set_data (G_OBJECT (item), "MailCheck", mc);
 
 	gtk_signal_connect_full (GTK_OBJECT (item), "activate",
 				 GTK_SIGNAL_FUNC (set_selection),
@@ -688,7 +689,7 @@ mailcheck_get_animation_menu (MailCheck *mc)
 	struct    dirent *e;
 	char      *dname = gnome_unconditional_pixmap_file ("mailcheck");
 	DIR       *dir;
-	char      *basename = NULL;
+	const char      *basename = NULL;
 	int       i = 0, select_item = 0;
 
 	mc->selected_pixmap_name = mc->mailcheck_text_only;
@@ -786,7 +787,7 @@ load_new_pixmap (MailCheck *mc)
 static void
 apply_properties_callback (GtkWidget *widget, gint page, gpointer data)
 {
-	char *text;
+	const char *text;
 	MailCheck *mc = (MailCheck *)data;
 	
 	if(page!=-1) return;
@@ -914,6 +915,7 @@ apply_properties_callback (GtkWidget *widget, gint page, gpointer data)
 		helper_whack_handle (mc->remote_handle);
 		mc->remote_handle = NULL;
 	}
+	applet_save_prefs(mc);
 }
 
 static void
@@ -941,7 +943,7 @@ make_remote_widgets_sensitive(MailCheck *mc)
 static void 
 set_mailbox_selection (GtkWidget *widget, gpointer data)
 {
-	MailCheck *mc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	MailCheck *mc = g_object_get_data(G_OBJECT(widget), "MailCheck");
 	mc->mailbox_type_temp = GPOINTER_TO_INT(data);
         
         make_remote_widgets_sensitive(mc);
@@ -972,33 +974,33 @@ mailbox_properties_page(MailCheck *mc)
 	l2 = gtk_menu_new();
 	item = gtk_menu_item_new_with_label(_("Local mailspool")); 
 	gtk_widget_show(item);
-	gtk_object_set_user_data(GTK_OBJECT(item), mc);
-	gtk_signal_connect (GTK_OBJECT(item), "activate", 
-			    GTK_SIGNAL_FUNC(set_mailbox_selection), 
+	g_object_set_data(G_OBJECT(item), "MailCheck", mc);
+	g_signal_connect (G_OBJECT(item), "activate", 
+			    G_CALLBACK(set_mailbox_selection), 
 			    GINT_TO_POINTER(MAILBOX_LOCAL));
 	gtk_menu_append(GTK_MENU(l2), item);
 
 	item = gtk_menu_item_new_with_label(_("Local maildir")); 
 	gtk_widget_show(item);
-	gtk_object_set_user_data(GTK_OBJECT(item), mc);
-	gtk_signal_connect (GTK_OBJECT(item), "activate", 
-			    GTK_SIGNAL_FUNC(set_mailbox_selection), 
+	g_object_set_data(G_OBJECT(item), "MailCheck", mc);
+	g_signal_connect (G_OBJECT(item), "activate", 
+			    G_CALLBACK(set_mailbox_selection), 
 			    GINT_TO_POINTER(MAILBOX_LOCALDIR));
 	gtk_menu_append(GTK_MENU(l2), item);
 
 	item = gtk_menu_item_new_with_label(_("Remote POP3-server")); 
 	gtk_widget_show(item);
-	gtk_object_set_user_data(GTK_OBJECT(item), mc);
-	gtk_signal_connect (GTK_OBJECT(item), "activate", 
-			    GTK_SIGNAL_FUNC(set_mailbox_selection), 
+	g_object_set_data(G_OBJECT(item), "MailCheck", mc);
+	g_signal_connect (G_OBJECT(item), "activate", 
+			    G_CALLBACK(set_mailbox_selection), 
 			    GINT_TO_POINTER(MAILBOX_POP3));
         
 	gtk_menu_append(GTK_MENU(l2), item);
 	item = gtk_menu_item_new_with_label(_("Remote IMAP-server")); 
 	gtk_widget_show(item);
-	gtk_object_set_user_data(GTK_OBJECT(item), mc);
-	gtk_signal_connect (GTK_OBJECT(item), "activate", 
-			    GTK_SIGNAL_FUNC(set_mailbox_selection), 
+	g_object_set_data(G_OBJECT(item), "MailCheck", mc);
+	g_signal_connect (G_OBJECT(item), "activate", 
+			    G_CALLBACK(set_mailbox_selection), 
 			    GINT_TO_POINTER(MAILBOX_IMAP));
 	gtk_menu_append(GTK_MENU(l2), item);
 	
@@ -1024,8 +1026,8 @@ mailbox_properties_page(MailCheck *mc)
 
 	mc->mailfile_entry = l = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY (l));
 	gtk_entry_set_text(GTK_ENTRY(l), mc->mail_file);
-	gtk_signal_connect(GTK_OBJECT(l), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1041,8 +1043,8 @@ mailbox_properties_page(MailCheck *mc)
   	gtk_widget_show(l);
 	gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);      
 	
-	gtk_signal_connect(GTK_OBJECT(l), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 	
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1059,8 +1061,8 @@ mailbox_properties_page(MailCheck *mc)
 	gtk_widget_show(l);
 	gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);      
   
-	gtk_signal_connect(GTK_OBJECT(l), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 
 	mc->remote_password_label = l = gtk_label_new(_("Password:"));
 	gtk_widget_show(l);
@@ -1073,8 +1075,8 @@ mailbox_properties_page(MailCheck *mc)
 	gtk_widget_show(l);
 	gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);      
 	
-	gtk_signal_connect(GTK_OBJECT(l), "changed",
-                     GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "changed",
+                     G_CALLBACK(property_box_changed), mc);
 
 	hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1091,8 +1093,8 @@ mailbox_properties_page(MailCheck *mc)
         gtk_widget_show(l);
         gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
   
-        gtk_signal_connect(GTK_OBJECT(l), "changed",
-                           GTK_SIGNAL_FUNC(property_box_changed), mc);
+        g_signal_connect(G_OBJECT(l), "changed",
+                           G_CALLBACK(property_box_changed), mc);
  
         hbox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -1108,8 +1110,8 @@ mailbox_properties_page(MailCheck *mc)
   	gtk_widget_show(l);
 	gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);      
 	
-	gtk_signal_connect(GTK_OBJECT(l), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "changed",
+			   G_CALLBACK(property_box_changed), mc);
   
 	make_remote_widgets_sensitive(mc);
 	
@@ -1141,8 +1143,8 @@ mailcheck_properties_page (MailCheck *mc)
 
 	l = gtk_check_button_new_with_label(_("Before each update:"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l), mc->pre_check_enabled);
-	gtk_signal_connect(GTK_OBJECT(l), "toggled",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "toggled",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_widget_show(l);
 	mc->pre_check_cmd_check = l;
 	
@@ -1154,16 +1156,16 @@ mailcheck_properties_page (MailCheck *mc)
 	if(mc->pre_check_cmd)
 		gtk_entry_set_text(GTK_ENTRY(mc->pre_check_cmd_entry), 
 				   mc->pre_check_cmd);
-	gtk_signal_connect(GTK_OBJECT(mc->pre_check_cmd_entry), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(mc->pre_check_cmd_entry), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_widget_show(mc->pre_check_cmd_entry);
 	gtk_table_attach_defaults (GTK_TABLE (table), mc->pre_check_cmd_entry,
 				   1, 2, 0, 1);
 
 	l = gtk_check_button_new_with_label (_("When new mail arrives:"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l), mc->newmail_enabled);
-	gtk_signal_connect(GTK_OBJECT(l), "toggled",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "toggled",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_widget_show(l);
 	gtk_table_attach (GTK_TABLE (table), l, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 	mc->newmail_cmd_check = l;
@@ -1173,16 +1175,16 @@ mailcheck_properties_page (MailCheck *mc)
 		gtk_entry_set_text(GTK_ENTRY(mc->newmail_cmd_entry),
 				   mc->newmail_cmd);
 	}
-	gtk_signal_connect(GTK_OBJECT (mc->newmail_cmd_entry), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT (mc->newmail_cmd_entry), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_widget_show(mc->newmail_cmd_entry);
 	gtk_table_attach_defaults (GTK_TABLE (table), mc->newmail_cmd_entry,
 				    1, 2, 1, 2);
 
         l = gtk_check_button_new_with_label (_("When clicked:"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l), mc->clicked_enabled);
-	gtk_signal_connect(GTK_OBJECT(l), "toggled",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(l), "toggled",
+			   G_CALLBACK(property_box_changed), mc);
         gtk_widget_show(l);
 	gtk_table_attach (GTK_TABLE (table), l, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
 	mc->clicked_cmd_check = l;
@@ -1192,8 +1194,8 @@ mailcheck_properties_page (MailCheck *mc)
 		gtk_entry_set_text(GTK_ENTRY(mc->clicked_cmd_entry), 
 				   mc->clicked_cmd);
         }
-        gtk_signal_connect(GTK_OBJECT(mc->clicked_cmd_entry), "changed",
-                           GTK_SIGNAL_FUNC(property_box_changed), mc);
+        g_signal_connect(G_OBJECT(mc->clicked_cmd_entry), "changed",
+                           G_CALLBACK(property_box_changed), mc);
         gtk_widget_show(mc->clicked_cmd_entry);
 	gtk_table_attach_defaults (GTK_TABLE (table), mc->clicked_cmd_entry,
 				   1, 2, 2, 3);
@@ -1208,10 +1210,10 @@ mailcheck_properties_page (MailCheck *mc)
 
 	freq_a = gtk_adjustment_new((float)((mc->update_freq/1000)/60), 0, 1440, 1, 5, 5);
 	mc->min_spin = gtk_spin_button_new( GTK_ADJUSTMENT (freq_a), 1, 0);
-	gtk_signal_connect(GTK_OBJECT(freq_a), "value_changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
-	gtk_signal_connect(GTK_OBJECT(mc->min_spin), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(freq_a), "value_changed",
+			   G_CALLBACK(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(mc->min_spin), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_box_pack_start (GTK_BOX (hbox), mc->min_spin,  FALSE, FALSE, 0);
 	gtk_widget_show(mc->min_spin);
 	
@@ -1221,10 +1223,10 @@ mailcheck_properties_page (MailCheck *mc)
 	
 	freq_a = gtk_adjustment_new((float)((mc->update_freq/1000)%60), 0, 59, 1, 5, 5);
 	mc->sec_spin  = gtk_spin_button_new (GTK_ADJUSTMENT (freq_a), 1, 0);
-	gtk_signal_connect(GTK_OBJECT(freq_a), "value_changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
-	gtk_signal_connect(GTK_OBJECT(mc->sec_spin), "changed",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(freq_a), "value_changed",
+			   G_CALLBACK(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(mc->sec_spin), "changed",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_box_pack_start (GTK_BOX (hbox), mc->sec_spin,  FALSE, FALSE, 0);
 	gtk_widget_show(mc->sec_spin);
 	
@@ -1234,8 +1236,8 @@ mailcheck_properties_page (MailCheck *mc)
 	
 	mc->play_sound_check = gtk_check_button_new_with_label(_("Play a sound when new mail arrives"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mc->play_sound_check), mc->play_sound);
-	gtk_signal_connect(GTK_OBJECT(mc->play_sound_check), "toggled",
-			   GTK_SIGNAL_FUNC(property_box_changed), mc);
+	g_signal_connect(G_OBJECT(mc->play_sound_check), "toggled",
+			   G_CALLBACK(property_box_changed), mc);
 	gtk_widget_show(mc->play_sound_check);
 	gtk_box_pack_start(GTK_BOX (vbox), mc->play_sound_check, FALSE, FALSE, 0);
 
@@ -1255,6 +1257,7 @@ mailcheck_properties_page (MailCheck *mc)
 static void
 phelp_cb (GtkWidget *w, gint tab, gpointer data)
 {
+#ifdef FIXME
 	GnomeHelpMenuEntry help_entry = { "mailcheck_applet", NULL };
 
 	char *das_names[] =  { "index.html#MAILCHECK-PREFS",
@@ -1262,10 +1265,11 @@ phelp_cb (GtkWidget *w, gint tab, gpointer data)
 
 	help_entry.path = das_names[((MailCheck *)data)->type];
 	gnome_help_display(NULL, &help_entry);
+#endif
 }	
 
 static void
-mailcheck_properties (AppletWidget *applet, gpointer data)
+mailcheck_properties (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
 	GtkWidget *p;
 
@@ -1291,73 +1295,81 @@ mailcheck_properties (AppletWidget *applet, gpointer data)
 	gnome_property_box_append_page (GNOME_PROPERTY_BOX(mc->property_window),
 					p, gtk_label_new (_("Mailbox")));
 
-	gtk_signal_connect (GTK_OBJECT (mc->property_window), "apply",
-			    GTK_SIGNAL_FUNC(apply_properties_callback), mc);
-	gtk_signal_connect (GTK_OBJECT (mc->property_window), "destroy",
-			    GTK_SIGNAL_FUNC(close_callback), mc);
-	gtk_signal_connect (GTK_OBJECT (mc->property_window), "help",
-			    GTK_SIGNAL_FUNC(phelp_cb), mc);
+	g_signal_connect (G_OBJECT (mc->property_window), "apply",
+			    G_CALLBACK(apply_properties_callback), mc);
+	g_signal_connect (G_OBJECT (mc->property_window), "destroy",
+			    G_CALLBACK(close_callback), mc);
+	g_signal_connect (G_OBJECT (mc->property_window), "help",
+			    G_CALLBACK(phelp_cb), mc);
 
 	gtk_widget_show (mc->property_window);
 }
 
 static void
-check_callback (AppletWidget *applet, gpointer data)
+check_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
 	MailCheck *mc = data;
 
 	mail_check_timeout(mc);
 }
 
-
-static gint
-applet_save_session(GtkWidget *w,
-		    const char *privcfgpath,
-		    const char *globcfgpath,
-		    gpointer data)
+static void
+applet_load_prefs(MailCheck *mc)
 {
-	MailCheck *mc = data;
+	gchar *query;
 
-	gnome_config_push_prefix(privcfgpath);
-	gnome_config_set_string("mail/animation_file",
-                          mc->animation_file?mc->animation_file:"");
-	gnome_config_set_int("mail/update_frequency", mc->update_freq);
-	gnome_config_set_string("mail/exec_command",
-				mc->pre_check_cmd?mc->pre_check_cmd:"");
-	gnome_config_set_bool("mail/exec_enabled",mc->pre_check_enabled);
-	gnome_config_set_string("mail/newmail_command",
-				mc->newmail_cmd?mc->newmail_cmd:"");
-	gnome_config_set_bool("mail/newmail_enabled",mc->newmail_enabled);
-	gnome_config_set_string("mail/clicked_command",
-				mc->clicked_cmd?mc->clicked_cmd:"");
-	gnome_config_set_bool("mail/clicked_enabled",mc->clicked_enabled);
-	gnome_config_set_string("mail/mail_file",
-				mc->mail_file?mc->mail_file:"");
-        gnome_config_private_set_string("mail/remote_server", 
-				mc->remote_server?mc->remote_server:"");
-        gnome_config_private_set_string("mail/remote_username", 
-				mc->remote_username?mc->remote_username:"");
-        gnome_config_private_set_string("mail/remote_password", 
-				mc->remote_password?mc->remote_password:"");
-        gnome_config_set_string("mail/remote_folder",
-                                mc->remote_folder?mc->remote_folder:"");
-        gnome_config_private_set_string("mail/pre_remote_command", 
-				mc->pre_remote_command ? mc->pre_remote_command : "");
-        gnome_config_set_int("mail/mailbox_type", (int) mc->mailbox_type);
-	gnome_config_set_bool("mail/play_sound", mc->play_sound);
-
-	gnome_config_pop_prefix();
-
-	gnome_config_sync();
-	gnome_config_drop_all();
-
-	return FALSE;
+	query = gnome_unconditional_pixmap_file("mailcheck/email.png");
+	mc->animation_file = panel_applet_gconf_get_string(mc->applet, "animation-file", NULL);
+	if(!mc->animation_file || strcmp(mc->animation_file, "none")) {
+		g_free(mc->animation_file);
+		mc->animation_file = g_strdup(query);
+	}
+	g_free(query);
+	mc->update_freq = panel_applet_gconf_get_int(mc->applet, "update-frequency", NULL);
+	mc->pre_check_cmd = panel_applet_gconf_get_string(mc->applet, "exec-command", NULL);
+	mc->pre_check_enabled = panel_applet_gconf_get_bool(mc->applet, "exec-enabled", NULL);
+	mc->newmail_cmd = panel_applet_gconf_get_string(mc->applet, "newmail-command", NULL);
+	mc->newmail_enabled = panel_applet_gconf_get_bool(mc->applet, "newmail-enabled", NULL);
+	mc->clicked_cmd = panel_applet_gconf_get_string(mc->applet, "clicked-command", NULL);
+	mc->clicked_enabled = panel_applet_gconf_get_bool(mc->applet, "clicked-enabled", NULL);
+	mc->remote_server = panel_applet_gconf_get_string(mc->applet, "remote-server", NULL);
+	mc->pre_remote_command = panel_applet_gconf_get_string(mc->applet, "pre-remote-command", NULL);
+	mc->remote_username = panel_applet_gconf_get_string(mc->applet, "remote-username", NULL);
+	if(!mc->remote_username || strcmp(mc->remote_username, "none")) {
+		g_free(mc->remote_username);
+		mc->remote_username = g_strdup(g_getenv("USER"));
+	}
+	mc->remote_password = panel_applet_gconf_get_string(mc->applet, "remote-password", NULL);
+	mc->remote_folder = panel_applet_gconf_get_string(mc->applet, "remote-folder", NULL);
+	mc->mailbox_type = panel_applet_gconf_get_int(mc->applet, "mailbox-type", NULL);
+	mc->play_sound = panel_applet_gconf_get_bool(mc->applet, "play-sound", NULL);
 }
 
 static void
-mailcheck_about(AppletWidget *a_widget, gpointer a_data)
+applet_save_prefs(MailCheck *mc)
 {
-	MailCheck *mc = a_data;
+	panel_applet_gconf_set_string(mc->applet, "animation-file", mc->animation_file, NULL);
+	panel_applet_gconf_set_int(mc->applet, "update-frequency", mc->update_freq, NULL);
+	panel_applet_gconf_set_string(mc->applet, "exec-command", mc->pre_check_cmd, NULL);
+	panel_applet_gconf_set_bool(mc->applet, "exec-enabled", mc->pre_check_enabled, NULL);
+	panel_applet_gconf_set_string(mc->applet, "newmail-command", mc->newmail_cmd, NULL);
+	panel_applet_gconf_set_bool(mc->applet, "newmail-enabled", mc->newmail_enabled, NULL);
+	panel_applet_gconf_set_string(mc->applet, "clicked-command", mc->clicked_cmd, NULL);
+	panel_applet_gconf_set_bool(mc->applet, "clicked-enabled", mc->clicked_enabled, NULL);
+	panel_applet_gconf_set_string(mc->applet, "mail-file", mc->mail_file, NULL);
+	panel_applet_gconf_set_string(mc->applet, "remote-server", mc->remote_server, NULL);
+	panel_applet_gconf_set_string(mc->applet, "remote-username", mc->remote_username, NULL);
+	panel_applet_gconf_set_string(mc->applet, "remote-password", mc->remote_password, NULL);
+	panel_applet_gconf_set_string(mc->applet, "remote-folder", mc->remote_folder, NULL);
+	panel_applet_gconf_set_string(mc->applet, "pre-remote-command", mc->pre_remote_command, NULL);
+	panel_applet_gconf_set_int(mc->applet, "mailbox-type", (gint)mc->mailbox_type, NULL);
+	panel_applet_gconf_set_bool(mc->applet, "play-sound", mc->play_sound, NULL);
+}
+
+static void
+mailcheck_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+{
+	MailCheck *mc = data;
 	static const gchar     *authors [] =
 	{
 		"Miguel de Icaza <miguel@kernel.org>",
@@ -1374,23 +1386,27 @@ mailcheck_about(AppletWidget *a_widget, gpointer a_data)
 		return;
 	}
 	
-	mc->about = gnome_about_new ( _("Mail check Applet"), "1.1",
-				      _("(c) 1998-2000 the Free Software Foundation"),
-				      authors,
-				      _("Mail check notifies you when new mail is on your mailbox"),
-				      NULL);
+	mc->about = gnome_about_new (_("Mail check Applet"), "1.1",
+				     _("(c) 1998-2000 the Free Software Foundation"),
+				     _("Mail check notifies you when new mail arrives in your mailbox"),
+				     authors,
+				     NULL,
+				     NULL,
+				     NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (mc->about),
 				"mailcheck", "Mailcheck");
+#ifdef FIXME
 	gnome_window_icon_set_from_file (GTK_WINDOW (mc->about),
 					 GNOME_ICONDIR"/gnome-mailcheck.png");
-	gtk_signal_connect( GTK_OBJECT(mc->about), "destroy",
-			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &mc->about );
+#endif
+	g_signal_connect( G_OBJECT(mc->about), "destroy",
+			    G_CALLBACK(gtk_widget_destroyed), &mc->about );
 	gtk_widget_show(mc->about);
 }
 
 /*this is when the panel size changes */
 static void
-applet_change_pixel_size(GtkWidget * w, int size, gpointer data)
+applet_change_pixel_size(PanelApplet * w, gint size, gpointer data)
 {
 	MailCheck *mc = data;
 	char *fname;
@@ -1402,7 +1418,7 @@ applet_change_pixel_size(GtkWidget * w, int size, gpointer data)
 	fname = mail_animation_filename (mc);
 
 	gtk_drawing_area_size (GTK_DRAWING_AREA(mc->da),size,size);
-	gtk_widget_set_usize (GTK_WIDGET(mc->da), size, size);
+	gtk_widget_set_size_request (GTK_WIDGET(mc->da), size, size);
 	
 	if (!fname)
 		return;
@@ -1412,27 +1428,41 @@ applet_change_pixel_size(GtkWidget * w, int size, gpointer data)
 }
 
 static void
-help_callback (AppletWidget *widget, gpointer data)
+help_callback (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
+#ifdef FIXME
 	GnomeHelpMenuEntry help_ref = { "mailcheck_applet", "index.html"};
 	gnome_help_display (NULL, &help_ref);
+#endif
 }
 
-GtkWidget *
-make_mailcheck_applet(const gchar *goad_id)
+static const BonoboUIVerb mailcheck_menu_verbs [] = {
+	BONOBO_UI_UNSAFE_VERB ("Properties", mailcheck_properties),
+	BONOBO_UI_UNSAFE_VERB ("Help",       help_callback),
+	BONOBO_UI_UNSAFE_VERB ("About",      mailcheck_about),
+	BONOBO_UI_UNSAFE_VERB ("Check",      check_callback),
+        BONOBO_UI_VERB_END
+};
+
+static const char mailcheck_menu_xml [] =
+	"<popup name=\"button3\">\n"
+	"   <menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"Properties ...\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
+	"   <menuitem name=\"Help Item\" verb=\"Help\" _label=\"Help\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
+	"   <menuitem name=\"About Item\" verb=\"About\" _label=\"About ...\"\n"
+	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
+	"   <menuitem name=\"Check Item\" verb=\"Check\" _label=\"Check for mail\"/>\n"
+	"</popup>\n";
+	
+gboolean
+fill_mailcheck_applet(PanelApplet *applet)
 {
 	GtkWidget *mailcheck;
 	MailCheck *mc;
-	char *emailfile;
-	char *query;
-
-	applet = applet_widget_new(goad_id);
-	if (!applet) {
-		g_warning(_("Can't create applet!\n"));
-		return NULL;
-	}
 
 	mc = g_new0(MailCheck, 1);
+	mc ->applet = applet;
 	mc->animation_file = NULL;
 	mc->property_window = NULL;
 	mc->anim_changed = FALSE;
@@ -1442,101 +1472,46 @@ make_mailcheck_applet(const gchar *goad_id)
 
 	/*initial state*/
 	mc->report_mail_mode = REPORT_MAIL_USE_ANIMATION;
-
-	gnome_config_push_prefix(APPLET_WIDGET(applet)->privcfgpath);
-
-	mc->mail_file = gnome_config_get_string("mail/mail_file");
+	
+	mc->mail_file = NULL;
 
 	if (mc->mail_file == NULL) {
-		mc->mail_file = g_getenv ("MAIL");
-		if (mc->mail_file == NULL) {
-			char *user = g_getenv ("USER");
+		const char *mail_file = g_getenv ("MAIL");
+		if (mail_file == NULL) {
+			const char *user = g_getenv ("USER");
 			if (user == NULL)
-				return NULL;
+				return FALSE;
 
 			mc->mail_file = g_strdup_printf ("/var/spool/mail/%s",
 							 user);
 		} else
-			mc->mail_file = g_strdup (mc->mail_file);
+			mc->mail_file = g_strdup (mail_file);
 	}
 
-	emailfile = gnome_unconditional_pixmap_file("mailcheck/email.png");
-	query = g_strconcat("mail/animation_file=",emailfile,NULL);
-	g_free(emailfile);
-	mc->animation_file = gnome_config_get_string(query);
-	g_free(query);
-
-	mc->update_freq = gnome_config_get_int("mail/update_frequency=120000");
-		
-	mc->pre_check_cmd = gnome_config_get_string("mail/exec_command");
-	mc->pre_check_enabled = gnome_config_get_bool("mail/exec_enabled=0");
-
-	mc->newmail_cmd = gnome_config_get_string("mail/newmail_command");
-	mc->newmail_enabled = gnome_config_get_bool("mail/newmail_enabled=0");
-
-        mc->clicked_cmd = gnome_config_get_string("mail/clicked_command");
-	mc->clicked_enabled = gnome_config_get_bool("mail/clicked_enabled=0");
-
-	mc->remote_server = gnome_config_private_get_string("mail/remote_server=mail");
-	mc->pre_remote_command = gnome_config_private_get_string("mail/pre_remote_command=");
-	
-	query = g_strconcat("mail/remote_username=", g_getenv("USER"), NULL);
-	mc->remote_username = gnome_config_private_get_string(query);
-	g_free(query);
-
-	mc->remote_password = gnome_config_private_get_string("mail/remote_password");
-        mc->remote_folder = gnome_config_get_string("mail/remote_folder");
-	mc->mailbox_type = gnome_config_get_int("mail/mailbox_type=0");
-
-	mc->play_sound = gnome_config_get_bool("mail/play_sound=false");
-
-	gnome_config_pop_prefix();
+	panel_applet_add_preferences (applet, "/schemas/apps/mailcheck-applet/prefs", NULL);
+	applet_load_prefs(mc);
 
 	mc->mailcheck_text_only = _("Text only");
-	
-	mc->size = PIXEL_SIZE_STANDARD;
 
-	gtk_signal_connect(GTK_OBJECT(applet), "change_pixel_size",
-			   GTK_SIGNAL_FUNC(applet_change_pixel_size),
-			   mc);
+	mc->size = GNOME_Vertigo_PANEL_MEDIUM;
+
+	g_signal_connect(G_OBJECT(applet), "change_size",
+			 G_CALLBACK(applet_change_pixel_size),
+			 mc);
 
 	mailcheck = create_mail_widgets (mc);
 	gtk_widget_show(mailcheck);
-	applet_widget_add (APPLET_WIDGET (applet), mailcheck);
 
-        gtk_signal_connect(GTK_OBJECT(mc->ebox), "button_press_event",
-                           GTK_SIGNAL_FUNC(exec_clicked_cmd), mc);
+	gtk_container_add (GTK_CONTAINER (applet), mailcheck);
 
-	gtk_signal_connect(GTK_OBJECT(applet),"save_session",
-			   GTK_SIGNAL_FUNC(applet_save_session),
-			   mc);
+	g_signal_connect(G_OBJECT(mc->ebox), "button_press_event",
+			 G_CALLBACK(exec_clicked_cmd), mc);
 
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "properties",
-					      GNOME_STOCK_MENU_PROP,
-					      _("Properties..."),
-					      mailcheck_properties,
-					      mc);
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "check_mail",
-					      GNOME_STOCK_MENU_MAIL,
-					      _("Check for mail"),
-					      check_callback,
-					      mc);
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "help",
-					      GNOME_STOCK_PIXMAP_HELP,
-					      _("Help"),
-					      help_callback,
-					      NULL);
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "about",
-					      GNOME_STOCK_MENU_ABOUT,
-					      _("About..."),
-					      mailcheck_about,
-					      mc);	
 
-	gtk_widget_show (applet);
+	panel_applet_setup_menu (applet, mailcheck_menu_xml, 
+			         mailcheck_menu_verbs, mc);
+	
+	gtk_widget_show (GTK_WIDGET (applet));
 
 	/*
 	 * check the mail right now, so we don't have to wait
@@ -1544,5 +1519,5 @@ make_mailcheck_applet(const gchar *goad_id)
 	 */
 	mail_check_timeout (mc);
 
-	return applet;
+	return(TRUE);
 }
