@@ -117,8 +117,7 @@ panel_applet_clean_gconf (AppletInfo *info)
         }
 
         g_free (temp_key);
-        g_slist_foreach (id_list, (GFunc) g_free, NULL);
-        g_slist_free (id_list);
+	panel_g_slist_deep_free (id_list);
 }
 
 /*destroy widgets and call the above cleanup function*/
@@ -127,24 +126,28 @@ panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
+	if (info->remove_idle != 0) {
+		g_source_remove (info->remove_idle);
+		info->remove_idle = 0;
+	}
+
 	panel_applet_clean_gconf (info);
 
 	applets = g_slist_remove (applets, info);
 
 	if (info->widget != NULL) {
+		GtkWidget *widget = info->widget;
 		if(info->type == APPLET_STATUS) {
 			status_applet_put_offscreen (info->data);
 		}
 		/* destroy will remove it from the panel */
-		gtk_widget_destroy (info->widget);
 		info->widget = NULL;
+		gtk_widget_destroy (widget);
 	}
 
 	info->data = NULL;
 
-	/*
-	 * FIXME: where are we supposed to free the structure?
-	 */
+	g_free (info);
 }
 
 static gboolean
@@ -152,6 +155,8 @@ applet_idle_remove (gpointer data)
 {
 	AppletInfo *info = data;
 
+	info->remove_idle = 0;
+
 	panel_applet_save_position (info, info->gconf_key);
 
 	if (info->type == APPLET_SWALLOW) {
@@ -169,7 +174,8 @@ static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
 {
-	g_idle_add (applet_idle_remove, info);
+	if (info->remove_idle == 0)
+		info->remove_idle = g_idle_add (applet_idle_remove, info);
 }
 
 static void
@@ -188,15 +194,15 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		} else if (strcmp (menu->name, "help_on_app") == 0) {
 			Launcher * launcher = menu->info->data;
 			if (launcher->ditem != NULL) {
+				GError *error = NULL;
 				const char *docpath =
 					gnome_desktop_item_get_string
 					    (launcher->ditem, "DocPath");
-				char *path =
-					panel_gnome_kde_help_path (docpath);
-				if (path != NULL) {
-					gnome_url_show (path, NULL);
-					/* FIXME: handle errors */
-					g_free (path);
+				if ( ! panel_show_gnome_kde_help (docpath, &error)) {
+					panel_error_dialog ("cannot_show_gnome_kde_help",
+							    _("<b>Cannot display help document</b>\n\n"
+							      "Details: %s"), error->message);
+					g_clear_error (&error);
 				}
 			}
 		}
@@ -427,7 +433,7 @@ setup_an_item(AppletUserMenu *menu,
 	/* if the item is a submenu and doesn't have it's menu
 	   created yet*/
 	} else if(!menu->submenu) {
-		menu->submenu = gtk_menu_new ();
+		menu->submenu = panel_menu_new ();
 	}
 
 	if(menu->submenu) {
@@ -489,7 +495,7 @@ add_to_submenus (AppletInfo *info,
 	}
 	
 	if (s_menu->submenu == NULL) {
-		s_menu->submenu = gtk_menu_new ();
+		s_menu->submenu = panel_menu_new ();
 		/*a more elegant way to do this should be done
 		  when I don't want to go to sleep */
 		if (s_menu->menuitem != NULL) {
@@ -547,7 +553,7 @@ applet_setup_panel_menu (gboolean is_basep)
 		setup_menuitem (menuitem, NULL, _("Panel"));
 	}
 
-	panel_menu = gtk_menu_new ();
+	panel_menu = panel_menu_new ();
 
 	make_panel_submenu (panel_menu, TRUE, is_basep);
 
@@ -563,7 +569,7 @@ panel_applet_create_menu (AppletInfo *info,
 	GtkWidget *menuitem;
 	GList     *l;
 
-	info->menu = g_object_ref (gtk_menu_new ());
+	info->menu = g_object_ref (panel_menu_new ());
 	gtk_object_sink (GTK_OBJECT (info->menu));
 
 	if (!commie_mode) {
@@ -680,7 +686,6 @@ applet_show_menu (AppletInfo     *info,
 	if (!GTK_WIDGET_REALIZED (info->menu))
 		gtk_widget_show (info->menu);
 
-	/* FIXME - off panel popups, should be automatic with new gtk-menu? */
 	gtk_menu_popup (GTK_MENU (info->menu),
 			NULL,
 			NULL,
@@ -690,33 +695,53 @@ applet_show_menu (AppletInfo     *info,
 			event->time);
 }
 
+static gboolean
+applet_do_popup_menu (GtkWidget      *widget,
+		      GdkEventButton *event,
+		      AppletInfo     *info)
+{
+	if (panel_applet_in_drag)
+		return FALSE;
+
+	switch (info->type) {
+	case APPLET_BONOBO:
+			break;
+	case APPLET_SWALLOW: {
+		GtkHandleBox *handle_box;
+
+		handle_box = GTK_HANDLE_BOX (((Swallow *)info->data)->handle_box);
+
+		if (handle_box->child_detached)
+			applet_show_menu (info, event);
+		}
+		break;
+	default:
+		applet_show_menu (info, event);
+		break;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+applet_popup_menu (GtkWidget      *widget,
+		   AppletInfo     *info)
+{
+	GdkEventButton event;
+
+	event.button = 3;
+	event.time = GDK_CURRENT_TIME;
+
+	return applet_do_popup_menu (widget, &event, info);
+}
+
 static gboolean
 applet_button_press (GtkWidget      *widget,
 		     GdkEventButton *event,
 		     AppletInfo     *info)
 {
 	if (event->button == 3) {
-		if (panel_applet_in_drag)
-			return FALSE;
-
-		switch (info->type) {
-		case APPLET_BONOBO:
-			break;
-		case APPLET_SWALLOW: {
-			GtkHandleBox *handle_box;
-
-			handle_box = GTK_HANDLE_BOX (((Swallow *)info->data)->handle_box);
-
-			if (handle_box->child_detached)
-				applet_show_menu (info, event);
-			}
-			break;
-		default:
-			applet_show_menu (info, event);
-			break;
-		}
-
-		return TRUE;
+		return applet_do_popup_menu (widget, event, info);
 	}
 
 	if (BUTTON_IS_WIDGET (widget))
@@ -728,10 +753,12 @@ applet_button_press (GtkWidget      *widget,
 static void
 applet_destroy (GtkWidget *w, AppletInfo *info)
 {
+	GtkWidget *old_widget;
 	GList *li;
-	
+
 	g_return_if_fail (info != NULL);
 
+	old_widget = info->widget;
 	info->widget = NULL;
 
 	if (info->type == APPLET_DRAWER) {
@@ -786,6 +813,11 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 	}
 	g_list_free (info->user_menu);
 	info->user_menu = NULL;
+
+	/* If this was not called from the panel_applet_clean
+	 * itself.  That is if the widget entry was still set */
+	if (old_widget != NULL)
+		panel_applet_clean (info);
 }
 
 static char *
@@ -913,8 +945,7 @@ panel_applet_load_list (AppletType   type,
 		panel_applet_load_from_unique_id (type, client, profile, (char *) l->data, use_default);
 
         g_free (temp_key);
-        g_slist_foreach (id_list, (GFunc) g_free, NULL);
-        g_slist_free (id_list);
+	panel_g_slist_deep_free (id_list);
 }
 
 void
@@ -953,7 +984,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 	gconf_client_set_string (client, temp_key, panel_applet_get_panel_id (applet_info), NULL);
 	g_free (temp_key);
 
-	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile, gconf_key, "right-stick", FALSE);
+	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile, gconf_key, "panel-right-stick", FALSE);
 	gconf_client_set_bool (client, temp_key, panel_applet_get_right_stick (applet_info), NULL);
 	g_free (temp_key);
 
@@ -992,8 +1023,7 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 	}
 
 	g_free (temp_key);
-	g_slist_foreach (id_list, (GFunc) g_free, NULL);
-	g_slist_free (id_list);
+	panel_g_slist_deep_free (id_list);
 
 	temp_key = panel_applet_get_full_gconf_key (applet_info->type, profile,
 						    applet_info->gconf_key, "object-type", FALSE);
@@ -1006,7 +1036,7 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 
 	switch (applet_info->type) {
 	case APPLET_BONOBO:
-		panel_applet_frame_save_to_gconf (PANEL_APPLET_FRAME (applet_info->data),
+		panel_applet_frame_save_to_gconf (PANEL_APPLET_FRAME (applet_info->widget),
 						  applet_info->gconf_key);
 		break;
 	case APPLET_DRAWER:
@@ -1120,25 +1150,29 @@ panel_applet_register (GtkWidget      *applet,
 				break;
 		if(!list) {
 			/*can't put it anywhere, clean up*/
-			gtk_widget_destroy(applet);
 			info->widget = NULL;
-			panel_applet_clean(info);
+			gtk_widget_destroy (applet);
+			panel_applet_clean (info);
 			g_warning(_("Can't find an empty spot"));
-			g_free (info);
 			return NULL;
 		}
 		panel = PANEL_WIDGET(list->data);
 	}
 
-	if(BUTTON_IS_WIDGET (applet) || !GTK_WIDGET_NO_WINDOW(applet))
+	if(BUTTON_IS_WIDGET (applet) || !GTK_WIDGET_NO_WINDOW(applet)) {
 		g_signal_connect(G_OBJECT(applet),
 				   "button_press_event",
 				   G_CALLBACK(applet_button_press),
 				   info);
+		g_signal_connect(G_OBJECT (applet),
+				   "popup_menu",
+                          	   G_CALLBACK(applet_popup_menu),
+				   info);
+	}
 
-	g_signal_connect(G_OBJECT(applet), "destroy",
-			   G_CALLBACK(applet_destroy),
-			   info);
+	g_signal_connect (G_OBJECT (applet), "destroy",
+			  G_CALLBACK (applet_destroy),
+			  info);
 
 	gtk_widget_show_all(applet);
 
