@@ -1,52 +1,62 @@
 #include <config.h>
+#include <math.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_affine.h>
-#include <libart_lgpl/art_filterlevel.h>
 #include "button-widget.h"
 #include "panel-widget.h"
 #include "basep-widget.h"
 #include "panel-types.h"
 #include "panel-util.h"
-#include "panel_config_global.h"
-#include "rgb-stuff.h"
+#include "panel-config-global.h"
+#include "panel-marshal.h"
 
-#undef BUTTON_WIDGET_DEBUG 
+#undef BUTTON_WIDGET_DEBUG
 
-#define BUTTON_WIDGET_DISPLACEMENT 5
+#define BUTTON_WIDGET_DISPLACEMENT 2
 
 extern GlobalConfig global_config;
 
-static void button_widget_class_init	(ButtonWidgetClass *klass);
-static void button_widget_instance_init	(ButtonWidget      *button);
-static void button_widget_size_request  (GtkWidget         *widget,
-					 GtkRequisition    *requisition);
-static void button_widget_size_allocate (GtkWidget         *widget,
-					 GtkAllocation     *allocation);
-static void button_widget_realize	(GtkWidget         *widget);
-static void button_widget_unrealize     (GtkWidget         *widget);
-static void button_widget_map           (GtkWidget         *widget);
-static void button_widget_unmap         (GtkWidget         *widget);
+static void     button_widget_class_init     (ButtonWidgetClass *klass);
+static void     button_widget_instance_init  (ButtonWidget      *button);
+static void     button_widget_size_request   (GtkWidget         *widget,
+					      GtkRequisition    *requisition);
+static void     button_widget_size_allocate  (GtkWidget         *widget,
+					      GtkAllocation     *allocation);
+static void     button_widget_realize        (GtkWidget         *widget);
+static gboolean button_widget_expose         (GtkWidget         *widget,
+					      GdkEventExpose    *event);
+static void     button_widget_destroy        (GtkObject         *obj);
+static gboolean button_widget_button_press   (GtkWidget         *widget,
+					      GdkEventButton	*event);
+static gboolean button_widget_enter_notify   (GtkWidget         *widget,
+					      GdkEventCrossing  *event);
+static gboolean button_widget_leave_notify   (GtkWidget         *widget,
+					      GdkEventCrossing  *event);
+static void     button_widget_button_pressed (GtkButton         *button);
+static void     button_widget_button_released (GtkButton         *button);
 
-static gboolean  button_widget_button_press	(GtkWidget         *widget,
-						 GdkEventButton    *event);
-static gboolean  button_widget_button_release(GtkWidget         *widget,
-					      GdkEventButton    *event);
-static gboolean  button_widget_enter_notify	(GtkWidget         *widget,
-						 GdkEventCrossing  *event);
-static gboolean  button_widget_leave_notify	(GtkWidget         *widget,
-						 GdkEventCrossing  *event);
-static void button_widget_pressed	(ButtonWidget *button);
-static void button_widget_unpressed	(ButtonWidget *button);
 
-/*list of all the button widgets*/
-static GList *buttons = NULL;
+static void     button_widget_push_move      (ButtonWidget      *button,
+                                              GtkDirectionType   dir);
+static void     button_widget_switch_move    (ButtonWidget      *button,
+                                              GtkDirectionType   dir);
+static void     button_widget_free_move      (ButtonWidget      *button,
+                                              GtkDirectionType   dir);
 
-static GtkWidgetClass *button_widget_parent_class;
+static GtkButtonClass *button_widget_parent_class;
+
+enum {
+	PUSH_MOVE_SIGNAL,
+	SWITCH_MOVE_SIGNAL,
+	FREE_MOVE_SIGNAL,
+	LAST_SIGNAL
+};
+
+static guint button_widget_signals[LAST_SIGNAL] = {0};
 
 GType
 button_widget_get_type (void)
@@ -67,93 +77,131 @@ button_widget_get_type (void)
 
 		};
 
-		object_type = g_type_register_static (GTK_TYPE_WIDGET, "ButtonWidget", &object_info, 0);
-		button_widget_parent_class = g_type_class_ref (GTK_TYPE_WIDGET);
+		object_type = g_type_register_static (GTK_TYPE_BUTTON, "ButtonWidget", &object_info, 0);
+		button_widget_parent_class = g_type_class_ref (GTK_TYPE_BUTTON);
 	}
 
 	return object_type;
 }
 
-enum {
-	CLICKED_SIGNAL,
-	PRESSED_SIGNAL,
-	UNPRESSED_SIGNAL,
-	LAST_SIGNAL
-};
-
-static guint button_widget_signals[LAST_SIGNAL] = {0};
-
 static void
 button_widget_class_init (ButtonWidgetClass *class)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (class);
+	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (class);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
+	GtkButtonClass *button_class = GTK_BUTTON_CLASS (class);
 
+	GtkBindingSet *binding_set;
 
-	button_widget_signals[CLICKED_SIGNAL] =
-		g_signal_new ("clicked",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (ButtonWidgetClass, clicked),
-                              NULL,
-                              NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-
-	button_widget_signals[PRESSED_SIGNAL] =
-		g_signal_new ("pressed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (ButtonWidgetClass, pressed),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-
-	button_widget_signals[UNPRESSED_SIGNAL] =
-		g_signal_new ("unpressed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (ButtonWidgetClass, unpressed),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-	
-	class->clicked = NULL;
-	class->pressed = button_widget_pressed;
-	class->unpressed = button_widget_unpressed;
-
+	gtk_object_class->destroy = button_widget_destroy;
+	  
 	widget_class->realize = button_widget_realize;
-	widget_class->unrealize = button_widget_unrealize;
-	widget_class->map = button_widget_map;
-	widget_class->unmap = button_widget_unmap;
 	widget_class->size_allocate = button_widget_size_allocate;
 	widget_class->size_request = button_widget_size_request;
 	widget_class->button_press_event = button_widget_button_press;
-	widget_class->button_release_event = button_widget_button_release;
 	widget_class->enter_notify_event = button_widget_enter_notify;
 	widget_class->leave_notify_event = button_widget_leave_notify;
-	
+	widget_class->expose_event = button_widget_expose;
+
+	button_class->pressed = button_widget_button_pressed;
+	button_class->released = button_widget_button_released;
+
+	class->push_move = button_widget_push_move;
+	class->switch_move = button_widget_switch_move;
+	class->free_move = button_widget_free_move;
+
+	button_widget_signals[PUSH_MOVE_SIGNAL] =
+		g_signal_new ("push_move",
+                              G_TYPE_FROM_CLASS (class),
+                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              G_STRUCT_OFFSET (ButtonWidgetClass, push_move),
+                              NULL,
+                              NULL,
+                              panel_marshal_VOID__ENUM,
+                              G_TYPE_NONE,
+                              1,
+                              GTK_TYPE_DIRECTION_TYPE);
+
+	button_widget_signals[SWITCH_MOVE_SIGNAL] =
+		g_signal_new ("switch_move",
+                              G_TYPE_FROM_CLASS (class),
+                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              G_STRUCT_OFFSET (ButtonWidgetClass, switch_move),
+                              NULL,
+                              NULL,
+                              panel_marshal_VOID__ENUM,
+                              G_TYPE_NONE,
+                              1,
+                              GTK_TYPE_DIRECTION_TYPE);
+
+	button_widget_signals[FREE_MOVE_SIGNAL] =
+		g_signal_new ("free_move",
+                              G_TYPE_FROM_CLASS (class),
+                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              G_STRUCT_OFFSET (ButtonWidgetClass, free_move),
+                              NULL,
+                              NULL,
+                              panel_marshal_VOID__ENUM,
+                              G_TYPE_NONE,
+                              1,
+                              GTK_TYPE_DIRECTION_TYPE);
+
+	binding_set = gtk_binding_set_by_class (class);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Up, GDK_SHIFT_MASK,
+                                      "push_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_UP);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Down, GDK_SHIFT_MASK,
+                                      "push_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_DOWN);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Left, GDK_SHIFT_MASK,
+                                      "push_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_LEFT);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Right, GDK_SHIFT_MASK,
+                                      "push_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_RIGHT);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Up, GDK_CONTROL_MASK,
+                                      "switch_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_UP);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Down, GDK_CONTROL_MASK,
+                                      "switch_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_DOWN);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Left, GDK_CONTROL_MASK,
+                                      "switch_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_LEFT);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Right, GDK_CONTROL_MASK,
+                                      "switch_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_RIGHT);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Up, GDK_MOD1_MASK,
+                                      "free_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_UP);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Down, GDK_MOD1_MASK,
+                                      "free_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_DOWN);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Left, GDK_MOD1_MASK,
+                                      "free_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_LEFT);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Right, GDK_MOD1_MASK,
+                                      "free_move", 1,
+                                      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_RIGHT);
 }
-
-
  
-static void
-setup_no_alpha(ButtonWidget *button)
-{
-	button->no_alpha = 0;
-}
-
-
 static void
 translate_to(GtkWidget *from, GtkWidget *to, int *x, int *y)
 {
 	while (from != to) {
-		if(!GTK_WIDGET_NO_WINDOW(from)) {
+		if (!GTK_WIDGET_NO_WINDOW (from)) {
 			*x += from->allocation.x;
 			*y += from->allocation.y;
 		}
@@ -266,7 +314,7 @@ button_widget_realize(GtkWidget *widget)
 {
 	GdkWindowAttr attributes;
 	gint attributes_mask;
-	ButtonWidget *button_widget;
+	GtkButton *button;
 	PanelWidget *panel;
 	GtkWidget *parent;
 	int x,y,w,h;
@@ -279,7 +327,7 @@ button_widget_realize(GtkWidget *widget)
 
 	calculate_overlay_geometry(panel, parent, widget, &x, &y, &w, &h);
 
-	button_widget = BUTTON_WIDGET (widget);
+	button = GTK_BUTTON (widget);
 
 	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
@@ -299,80 +347,45 @@ button_widget_realize(GtkWidget *widget)
 	attributes_mask = GDK_WA_X | GDK_WA_Y;
 
 	widget->window = gtk_widget_get_parent_window(widget);
-	gdk_window_ref(widget->window);
+	g_object_ref (G_OBJECT (widget->window));
       
-	button_widget->event_window = gdk_window_new (parent->window,
-						      &attributes,
-						      attributes_mask);
-	gdk_window_set_user_data (button_widget->event_window, widget);
+	button->event_window = gdk_window_new (parent->window,
+					       &attributes,
+					       attributes_mask);
+	gdk_window_set_user_data (button->event_window, widget);
 
 	widget->style = gtk_style_attach (widget->style, widget->window);
 }
 
 static void
-button_widget_unrealize (GtkWidget *widget)
+button_widget_destroy (GtkObject *obj)
 {
-	ButtonWidget *button_widget;
-  
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
+	ButtonWidget *button = BUTTON_WIDGET(obj);
 
-	button_widget = BUTTON_WIDGET (widget);
-	
-	gdk_window_set_user_data (button_widget->event_window, NULL);
-	gdk_window_destroy (button_widget->event_window);
-	button_widget->event_window = NULL;
-	
-	if (GTK_WIDGET_CLASS (button_widget_parent_class)->unrealize)
-		(* GTK_WIDGET_CLASS (button_widget_parent_class)->unrealize) (widget);
-}
+	if (button->pressed_timeout != 0)
+		gtk_timeout_remove (button->pressed_timeout);
+	button->pressed_timeout = 0;
 
-static void
-button_widget_map (GtkWidget *widget)
-{
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
-
-	if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
-		gdk_window_show (BUTTON_WIDGET (widget)->event_window);
-
-	GTK_WIDGET_CLASS (button_widget_parent_class)->map (widget);
-}
-
-static void
-button_widget_unmap (GtkWidget *widget)
-{
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
-
-	if (GTK_WIDGET_MAPPED (widget))
-		gdk_window_hide (BUTTON_WIDGET (widget)->event_window);
-
-	GTK_WIDGET_CLASS (button_widget_parent_class)->unmap (widget);
-}
-
-static void
-button_widget_destroy(GtkWidget *w, gpointer data)
-{
-	ButtonWidget *button = BUTTON_WIDGET(w);
-
-	if(button->pressed_timeout != 0)
-		gtk_timeout_remove(button->pressed_timeout);
-
-	if(button->pixbuf)
-		gdk_pixbuf_unref(button->pixbuf);
+	if (button->pixbuf)
+		g_object_unref (G_OBJECT (button->pixbuf));
 	button->pixbuf = NULL;
-	if(button->pixbuf_hc)
-		gdk_pixbuf_unref(button->pixbuf_hc);
-	button->pixbuf_hc = NULL;
-	if(button->cache)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
+	
+	if (button->scaled)
+		g_object_unref (G_OBJECT (button->scaled));
+	button->scaled = NULL;
+	
+	if (button->scaled_hc)
+		g_object_unref (G_OBJECT (button->scaled_hc));
+	button->scaled_hc = NULL;
+	
+	g_free (button->filename);
+	button->filename = NULL;
+	
+	g_free (button->text);
+	button->text = NULL;
 
-	g_free(button->filename);
-	g_free(button->text);
-
-	buttons = g_list_remove(buttons,button);
+	if (GTK_OBJECT_CLASS (button_widget_parent_class)->destroy)
+		GTK_OBJECT_CLASS (button_widget_parent_class)->destroy (obj);
 }
 
 static GdkPixbuf *
@@ -398,59 +411,6 @@ loadup_file(const char *file)
 	return pb;
 }
 
-#if 0
-/* #define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11) */
-#define INTENSITY(r, g, b) (((r)*77 + (g)*150 + (b)*28)>>8)
-
-/* saturation is 0-255, darken is 0-255 */
-static void
-do_saturate_darken (GdkPixbuf *dest, GdkPixbuf *src,
-		    int saturation, int darken)
-{
-	gint i, j;
-	gint width, height, has_alpha, srcrowstride, destrowstride;
-	guchar *target_pixels;
-	guchar *original_pixels;
-	guchar *pixsrc;
-	guchar *pixdest;
-	int intensity;
-	int alpha;
-	int negalpha;
-	int val;
-	guchar r,g,b;
-
-	has_alpha = gdk_pixbuf_get_has_alpha (src);
-	width = gdk_pixbuf_get_width (src);
-	height = gdk_pixbuf_get_height (src);
-	srcrowstride = gdk_pixbuf_get_rowstride (src);
-	destrowstride = gdk_pixbuf_get_rowstride (dest);
-	target_pixels = gdk_pixbuf_get_pixels (dest);
-	original_pixels = gdk_pixbuf_get_pixels (src);
-
-	for (i = 0; i < height; i++) {
-		pixdest = target_pixels + i*destrowstride;
-		pixsrc = original_pixels + i*srcrowstride;
-		for (j = 0; j < width; j++) {
-			r = *(pixsrc++);
-			g = *(pixsrc++);
-			b = *(pixsrc++);
-			intensity = INTENSITY(r,g,b);
-			negalpha = ((255 - saturation)*darken)>>8;
-			alpha = (saturation*darken)>>8;
-			val = (negalpha * intensity + alpha * r) >> 8;
-			*(pixdest++) = MIN(val, 255);
-			val = (negalpha * intensity + alpha * g) >> 8;
-			*(pixdest++) = MIN(val, 255);
-			val = (negalpha * intensity + alpha * b) >> 8;
-			*(pixdest++) = MIN(val, 255);
-			if (has_alpha)
-				*(pixdest++) = *(pixsrc++);
-		}
-	}
-}
-#undef INTENSITY
-#endif
-
 /* colorshift a pixbuf */
 static void
 do_colorshift (GdkPixbuf *dest, GdkPixbuf *src, int shift)
@@ -542,119 +502,117 @@ button_widget_set_dnd_highlight(ButtonWidget *button, gboolean highlight)
 
 	if(button->dnd_highlight != highlight) {
 		button->dnd_highlight = highlight;
-		if(button->cache)
-			gdk_pixmap_unref(button->cache);
-		button->cache = NULL;
-
-		panel_widget_draw_icon (PANEL_WIDGET(GTK_WIDGET(button)->parent), button);
+		gtk_widget_queue_draw (GTK_WIDGET (button));
 	}
 }
 
-void
-button_widget_draw(ButtonWidget *button, guchar *rgb, int rowstride)
+static gboolean
+button_widget_expose (GtkWidget         *widget,
+		      GdkEventExpose    *event)
 {
-	GtkWidget *widget, *pwidget;
-	PanelWidget *panel;
+	ButtonWidget *button_widget;
+	GtkButton *button;
+	GdkRectangle area, image_bound;
+	int off, size;
+	int x, y, w, h;
 	GdkPixbuf *pb = NULL;
-	int size, off;
+  
+	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
 
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
-	g_return_if_fail(rgb != NULL);
-	g_return_if_fail(rowstride >= 0);
+	button_widget = BUTTON_WIDGET (widget);
+	button = GTK_BUTTON (widget);
+	
+	if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget)) {
+		return FALSE;
+	}
 
-	widget = GTK_WIDGET(button);
-	panel = PANEL_WIDGET(widget->parent);
-	size = panel->sz;
+	size = widget->allocation.height;
 	/* offset for pressed buttons */
-	off = (button->in_button && button->pressed) ?
+	off = (button->in_button && button->button_down) ?
 		SCALE(BUTTON_WIDGET_DISPLACEMENT) : 0;
-	 
-	button->size = size;
 	
-	pwidget = widget->parent;
-
-	if(!global_config.highlight_when_over || !button->in_button) {
-		pb = button->pixbuf;
+	if (!global_config.highlight_when_over || !button->in_button) {
+		pb = button_widget->scaled;
 	} else {
-#ifdef BUTTON_WIDGET_DEBUG
-	printf ("Using highlighted pixbuf\n");
-#endif
-		pb = button->pixbuf_hc;
+		pb = button_widget->scaled_hc;
 	}
-	if(pb != NULL) {
-		double affine[6];
-		int w, h;
-		GdkPixbuf *scaled_pb;
-		GdkInterpType interp;
-
-		interp = GDK_INTERP_HYPER;
-
-		scaled_pb = scale_pixbuf_to_square (pb, size, &w, &h,
-						    interp);
-
-		art_affine_translate(affine, 
-				     off + (size-w)/2, 
-				     off + (size-h)/2);
-
-		transform_pixbuf(rgb,
-			         0, 0,
-			         size, size,
-			         rowstride,
-			         scaled_pb,
-			         affine, ART_FILTER_NEAREST, NULL);
-
-		gdk_pixbuf_unref (scaled_pb);
+	
+	w = gdk_pixbuf_get_width (pb);
+	h = gdk_pixbuf_get_height (pb);
+	x = widget->allocation.x + off + (widget->allocation.width - w)/2;
+	y = widget->allocation.y + off + (widget->allocation.height - h)/2;
+	
+	image_bound.x = x;
+	image_bound.y = y;      
+	image_bound.width = w;
+	image_bound.height = h;
+	
+	area = event->area;
+	
+	if (gdk_rectangle_intersect (&area, &widget->allocation, &area) &&
+	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))  {
+		gdk_pixbuf_render_to_drawable_alpha (pb,
+						     widget->window,
+						     image_bound.x - x, image_bound.y - y,
+						     image_bound.x, image_bound.y,
+						     image_bound.width, image_bound.height,
+						     GDK_PIXBUF_ALPHA_FULL,
+						     128,
+						     GDK_RGB_DITHER_NORMAL,
+						     0, 0);
 	}
-}
-
-/* draw the xlib part (arrow/text/dndhighlight) */
-
-void
-button_widget_draw_xlib(ButtonWidget *button, GdkPixmap *pixmap)
-{
-	GtkWidget *widget, *pwidget;
-	GdkGC *gc;
-	int size;
-	int off; 
-
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
-	g_return_if_fail(pixmap != NULL);
-
-	widget = GTK_WIDGET(button);
-	size = button->size;
-
-	/* offset for pressed buttons */
-	off = (button->in_button && button->pressed) ?
-		SCALE(BUTTON_WIDGET_DISPLACEMENT) : 0; 
-	gc = gdk_gc_new(pixmap);
 	
-	pwidget = widget->parent;
-	
-	if(button->arrow) {
+	if(button_widget->arrow) {
 		int i;
 		GdkPoint points[3];
-		draw_arrow(points,button->orient,size);
-		for(i=0;i<3;i++) {
-			points[i].x+=off;
-			points[i].y+=off;
+		draw_arrow (points, button_widget->orient, widget->allocation.height);
+		for (i = 0; i < 3; i++) {
+			points[i].x += off + widget->allocation.x;
+			points[i].y += off + widget->allocation.y;
 		}
-		gdk_gc_set_foreground(gc,&pwidget->style->white);
-		gdk_draw_polygon(pixmap,gc,TRUE,points,3);
-		gdk_gc_set_foreground(gc,&pwidget->style->black);
-		gdk_draw_polygon(pixmap,gc,FALSE,points,3);
+		gdk_draw_polygon (widget->window, widget->style->white_gc, TRUE, points, 3);
+		gdk_draw_polygon (widget->window, widget->style->black_gc, FALSE, points, 3);
 	}
 
-	if (button->dnd_highlight) {
-		gdk_gc_set_foreground(gc, &widget->style->black);
-		gdk_draw_rectangle(pixmap, gc, FALSE,
-				   0, 0,
+	if (button_widget->dnd_highlight) {
+		gdk_draw_rectangle(widget->window, widget->style->black_gc, FALSE,
+				   widget->allocation.x, widget->allocation.y,
 				   widget->allocation.width - 1,
 				   widget->allocation.height - 1);
 	}
+
+	if (GTK_WIDGET_HAS_FOCUS (widget)) {
+		gtk_paint_focus (widget->style, widget->window,
+				 GTK_WIDGET_STATE (widget),
+				 &event->area, widget, "button",
+				 widget->allocation.x + 1,
+				 widget->allocation.y + 1,
+				 widget->allocation.width - 3,
+				 widget->allocation.height - 3);
+	}
 	
-	gdk_gc_destroy(gc);
+	return FALSE;
+}
+
+static GdkPixbuf *
+make_hc_pixbuf(GdkPixbuf *pb)
+{
+	GdkPixbuf *new;
+	
+	if(!pb)
+		return NULL;
+#ifdef BUTTON_WIDGET_DEBUG
+	printf ("Creating highlight pixbuf\n");
+#endif
+	new = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pb),
+			      gdk_pixbuf_get_has_alpha (pb),
+			      gdk_pixbuf_get_bits_per_sample (pb),
+			      gdk_pixbuf_get_width (pb),
+			      gdk_pixbuf_get_height (pb));
+	do_colorshift (new, pb, 30);
+
+	return new;
 }
 
 static void
@@ -668,13 +626,41 @@ static void
 button_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
 	ButtonWidget *button_widget;
+	GtkButton *button;
+	int w, h;
+	double scale, scale_x, scale_y;
 
-	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BUTTON_IS_WIDGET (widget));
 
 	button_widget = BUTTON_WIDGET (widget);
+	button = GTK_BUTTON (widget);
 
 	widget->allocation = *allocation;
+
+	w = gdk_pixbuf_get_width (button_widget->pixbuf);
+	h = gdk_pixbuf_get_height (button_widget->pixbuf);
+
+	scale_x = (double) allocation->width / w;
+	scale_y = (double) allocation->height / h;
+
+	scale = MIN (scale_x, scale_y);
+
+	w *= scale;
+	h *= scale;
+
+	if (button_widget->scaled) {
+		g_object_unref (button_widget->scaled);
+	}
+
+	/* TODO: Don't scale if already the same size */
+	button_widget->scaled = gdk_pixbuf_scale_simple (button_widget->pixbuf,
+							 w, h, GDK_INTERP_BILINEAR);
+	if (button_widget->scaled_hc) {
+		g_object_unref (button_widget->scaled_hc);
+	}
+	
+	button_widget->scaled_hc = make_hc_pixbuf (button_widget->scaled);
+	
 	if (GTK_WIDGET_REALIZED (widget)) {
 		PanelWidget *panel;
 		int x,y,w,h;
@@ -683,37 +669,22 @@ button_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 		calculate_overlay_geometry (panel, panel->panel_parent,
 					    widget, &x, &y, &w, &h);
-		gdk_window_move_resize (button_widget->event_window,
-					x, y, w, h);
+		gdk_window_move_resize (button->event_window, x, y, w, h);
 	}
 }
 
-
-
 static void
 button_widget_instance_init (ButtonWidget *button)
 {
-	buttons = g_list_prepend(buttons,button);
-
-	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
-
 	button->pixbuf = NULL;
-	button->pixbuf_hc = NULL;
 	
-	button->pobject = 0;
 	button->arrow = 0;
 	button->orient = PANEL_ORIENT_UP;
 	
-	button->pressed = FALSE;
-	button->in_button = FALSE;
 	button->ignore_leave = FALSE;
 	button->dnd_highlight = FALSE;
-	
+
 	button->pressed_timeout = 0;
-	
-	g_signal_connect (G_OBJECT(button),"destroy",
-			  G_CALLBACK (button_widget_destroy),
-			  NULL);
 }
 
 static gboolean
@@ -721,22 +692,20 @@ pressed_timeout_func(gpointer data)
 {
 	ButtonWidget *button;
 
-	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (BUTTON_IS_WIDGET (data), FALSE);
 
 	button = BUTTON_WIDGET (data);
-	
+
 	button->pressed_timeout = 0;
-	
+
 	return FALSE;
 }
 
 static gboolean
 button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 {
-	ButtonWidget *button;
+ 	ButtonWidget *button;
 
-	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
@@ -745,52 +714,17 @@ button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 	if (button->pressed_timeout)
 		return TRUE;
 
-	if (event->button == 1) {
-		gtk_grab_add(widget);
-		button_widget_down (button);
-		button->pressed_timeout =
-			gtk_timeout_add(400, pressed_timeout_func, button);
-	}
-	return TRUE;
-}
-
-static gboolean
-button_widget_button_release (GtkWidget *widget, GdkEventButton *event)
-{
-	g_return_val_if_fail (widget != NULL, FALSE);
-	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
-
-	if (event->button == 1) {
-		ButtonWidget *button = BUTTON_WIDGET (widget);
-		gtk_grab_remove (widget);
-		button_widget_up (button);
-	}
-
-	return TRUE;
+	return GTK_WIDGET_CLASS (button_widget_parent_class)->button_press_event (widget, event);
 }
 
 static gboolean
 button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 {
-	GtkWidget *event_widget;
-
-	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
 
-	event_widget = gtk_get_event_widget ((GdkEvent*) event);
-
-	if ((event_widget == widget) &&
-	    (event->detail != GDK_NOTIFY_INFERIOR)) {
-		ButtonWidget *button = BUTTON_WIDGET (widget);
-		button->in_button = TRUE;
-		if (button->cache)
-			gdk_pixmap_unref (button->cache);
-		button->cache = NULL;
-		panel_widget_draw_icon (PANEL_WIDGET(widget->parent),
-					button);
-	}
+	GTK_WIDGET_CLASS (button_widget_parent_class)->enter_notify_event (widget, event);
+	if (GTK_BUTTON (widget)->in_button)
+		gtk_widget_queue_draw (widget);
 
 	return FALSE;
 }
@@ -799,168 +733,98 @@ static gboolean
 button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 {
 	GtkWidget *event_widget;
-	ButtonWidget *button;
+	ButtonWidget *button_widget;
+	GtkButton *button;
 
-	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
 	event_widget = gtk_get_event_widget ((GdkEvent*) event);
 
-	button = BUTTON_WIDGET (widget);
+	button_widget = BUTTON_WIDGET (widget);
+	button = GTK_BUTTON (widget);
 
 	if ((event_widget == widget) &&
 	    (event->detail != GDK_NOTIFY_INFERIOR) &&
-	    (!button->ignore_leave)) {
+	    (!button_widget->ignore_leave)) {
 		button->in_button = FALSE;
 		if (global_config.highlight_when_over) {
-			if (button->cache)
-				gdk_pixmap_unref (button->cache);
-			button->cache = NULL;
-			panel_widget_draw_icon (PANEL_WIDGET(widget->parent),
-						button);
+			gtk_widget_queue_draw (widget);
 		}
 	}
 
 	return FALSE;
 }
 
-
-void
-button_widget_clicked(ButtonWidget *button)
+static void
+button_widget_button_pressed (GtkButton *button)
 {
-	g_signal_emit (G_OBJECT(button),
-		       button_widget_signals[CLICKED_SIGNAL], 0);
+	ButtonWidget *button_widget;
+
+	g_return_if_fail (BUTTON_IS_WIDGET (button));
+
+	button_widget_parent_class->pressed (button);
+
+	button_widget = BUTTON_WIDGET (button);
+	button_widget->pressed_timeout =
+		gtk_timeout_add (400, pressed_timeout_func, button_widget);
+        gtk_widget_queue_draw (GTK_WIDGET (button));
 }
 
 static void
-button_widget_pressed(ButtonWidget *button)
+button_widget_button_released (GtkButton *button)
 {
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
+	g_return_if_fail (BUTTON_IS_WIDGET (button));
 
-	button->pressed = TRUE;
-	if(button->cache)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
-	setup_no_alpha(button);
-	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
-			       button);
+	button_widget_parent_class->released (button);
+        gtk_widget_queue_draw (GTK_WIDGET (button));
 }
-static void
-button_widget_unpressed(ButtonWidget *button)
-{
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
-
-	button->pressed = FALSE;
-	if(button->cache)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
-	setup_no_alpha(button);
-	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
-			       button);
-	if(button->in_button)
-		button_widget_clicked(button);
-}
-
-void
-button_widget_down(ButtonWidget *button)
-{
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
-
-	if(!button->pressed)
-		g_signal_emit(G_OBJECT(button),
-			      button_widget_signals[PRESSED_SIGNAL], 0);
-}
-void
-button_widget_up(ButtonWidget *button)
-{
-	g_return_if_fail(button != NULL);
-	g_return_if_fail(BUTTON_IS_WIDGET(button));
-
-	if(button->pressed)
-		g_signal_emit(G_OBJECT(button),
-			      button_widget_signals[UNPRESSED_SIGNAL], 0);
-}
-
-static GdkPixbuf *
-make_hc_pixbuf(GdkPixbuf *pb)
-{
-	GdkPixbuf *new;
-	if(!pb)
-		return NULL;
-#ifdef BUTTON_WIDGET_DEBUG
-	printf ("Creating highlight pixbuf\n");
-#endif
-	new = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pb),
-			     gdk_pixbuf_get_has_alpha(pb),
-			     gdk_pixbuf_get_bits_per_sample(pb),
-			     gdk_pixbuf_get_width(pb),
-			     gdk_pixbuf_get_height(pb));
-	do_colorshift(new, pb, 30);
-	/*do_saturate_darken (new, pb, (int)(1.00*255), (int)(1.15*255));*/
-
-	return new;
-}
-
 
 GtkWidget*
 button_widget_new(const char *filename,
 		  int size,
-		  int pobject,
 		  gboolean arrow,
 		  PanelOrient orient,
 		  const char *text)
 {
 	ButtonWidget *button;
 
-	g_return_val_if_fail(pobject >= 0, NULL);
-	g_return_val_if_fail(pobject < LAST_POBJECT, NULL);
-
-	button = BUTTON_WIDGET (gtk_type_new (button_widget_get_type ()));
+	button = BUTTON_WIDGET (g_object_new (button_widget_get_type (), NULL));
 	
 	button->pixbuf = loadup_file (filename);
-	button->pixbuf_hc = make_hc_pixbuf (button->pixbuf);
+	button->scaled = NULL;
+	button->scaled_hc = NULL;
 	button->filename = g_strdup (filename);
 	button->size = size;
-	button->pobject = pobject;
 	button->arrow = arrow ? 1 : 0;
 	button->orient = orient;
 	button->text = text ? g_strdup (text) : NULL;
 
-	setup_no_alpha(button);
-	
 	return GTK_WIDGET(button);
 }
 
 gboolean
 button_widget_set_pixmap(ButtonWidget *button, const char *pixmap, int size)
 {
-	g_return_val_if_fail(button != NULL, FALSE);
 	g_return_val_if_fail(BUTTON_IS_WIDGET(button), FALSE);
 
 	if (size < 0)
 		size = PANEL_WIDGET(GTK_WIDGET(button)->parent)->sz;
 	
 	if (button->pixbuf != NULL)
-		gdk_pixbuf_unref(button->pixbuf);
-	if (button->pixbuf_hc != NULL)
-		gdk_pixbuf_unref(button->pixbuf_hc);
+		g_object_unref (G_OBJECT (button->pixbuf));
 
 	button->pixbuf = loadup_file(pixmap);
-	button->pixbuf_hc = make_hc_pixbuf(button->pixbuf);
 
+	
 	g_free(button->filename);
 	button->filename = g_strdup(pixmap);
 	button->size = size;
-	if (button->cache != NULL)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
 
-	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
-			       button);
+	/* TODO: Is this guaranteed to trigger the size_allocate call to
+	   update scale + scale_hc???? */
+	gtk_widget_queue_resize (GTK_WIDGET (button));
+	gtk_widget_queue_draw (GTK_WIDGET (button));
 
 	if (button->pixbuf == NULL)
 		return FALSE;
@@ -971,55 +835,62 @@ button_widget_set_pixmap(ButtonWidget *button, const char *pixmap, int size)
 void
 button_widget_set_text(ButtonWidget *button, const char *text)
 {
-	g_return_if_fail(button != NULL);
 	g_return_if_fail(BUTTON_IS_WIDGET(button));
 
 	g_free(button->text);
 	button->text = text?g_strdup(text):NULL;
 
-	if (button->cache != NULL)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
-	
-	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
-			       button);
+	gtk_widget_queue_draw (GTK_WIDGET (button));
 }
 
 void
 button_widget_set_params(ButtonWidget *button,
-			 int pobject,
 			 gboolean arrow,
 			 PanelOrient orient)
 {
-	g_return_if_fail(button != NULL);
 	g_return_if_fail(BUTTON_IS_WIDGET(button));
-	g_return_if_fail(pobject >= 0);
-	g_return_if_fail(pobject < LAST_POBJECT);
 
-	button->pobject = pobject;
-	button->arrow = arrow?1:0;
+	button->arrow = arrow ? 1 : 0;
 	button->orient = orient;
 
-	if(button->cache)
-		gdk_pixmap_unref(button->cache);
-	button->cache = NULL;
-	setup_no_alpha(button);
-	
-	panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(button)->parent),
-			       button);
+	gtk_widget_queue_draw (GTK_WIDGET (button));
 }
 
-void
-button_widget_redo_all (void)
+static void
+button_widget_push_move (ButtonWidget      *button,
+                         GtkDirectionType   dir)
 {
-	GList *list;
+	PanelWidget *panel;
+	AppletData *ad;
 
-	for(list = buttons; list != NULL; list = list->next) {
-		ButtonWidget *button = list->data;
-		if(button->cache != NULL)
-			gdk_pixmap_unref(button->cache);
-		button->cache = NULL;
-		setup_no_alpha(button);
-		gtk_widget_queue_draw (GTK_WIDGET (button));
-	}
+	panel = PANEL_WIDGET(GTK_WIDGET (button)->parent);
+	ad = g_object_get_data (G_OBJECT (button), PANEL_APPLET_DATA);
+
+	panel_widget_push_applet (panel, ad, dir);
+}
+
+static void
+button_widget_switch_move (ButtonWidget      *button,
+                           GtkDirectionType   dir)
+{
+	PanelWidget *panel;
+	AppletData *ad;
+
+	panel = PANEL_WIDGET(GTK_WIDGET (button)->parent);
+	ad = g_object_get_data (G_OBJECT (button), PANEL_APPLET_DATA);
+
+	panel_widget_switch_applet (panel, ad, dir);
+}
+
+static void
+button_widget_free_move (ButtonWidget      *button,
+                         GtkDirectionType   dir)
+{
+	PanelWidget *panel;
+	AppletData *ad;
+
+	panel = PANEL_WIDGET(GTK_WIDGET (button)->parent);
+	ad = g_object_get_data (G_OBJECT (button), PANEL_APPLET_DATA);
+
+	panel_widget_free_move_applet (panel, ad, dir);
 }
