@@ -14,62 +14,71 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <string.h>
-#include <gnome.h>
 
-#include "panel-include.h"
-#include "icon-entry-hack.h"
+#include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
 
+#include "drawer.h"
+
+#include "applet.h"
+#include "button-widget.h"
+#include "drawer-widget.h"
+#include "panel-config-global.h"
+#include "panel-gconf.h"
+#include "panel-util.h"
+#include "session.h"
 #include "xstuff.h"
 
-extern GSList *applets;
-extern GSList *applets_last;
-extern int applet_count;
 extern GlobalConfig global_config;
 extern gboolean commie_mode;
 
-extern int applets_to_sync;
 extern int panels_to_sync;
-extern int need_complete_save;
 
 extern GtkTooltips *panel_tooltips;
+extern GSList *panels;
 
-extern GSList *panel_list;
+#undef DRAWER_DEBUG
 
 static void
 properties_apply_callback(gpointer data)
 {
 	Drawer       *drawer = data;
 
-	GtkWidget    *pixentry = gtk_object_get_data(GTK_OBJECT(drawer->properties),
-						     "pixmap");
-	GtkWidget    *tipentry = gtk_object_get_data(GTK_OBJECT(drawer->properties),
-						     "tooltip");
+	GtkWidget    *pixentry = g_object_get_data (G_OBJECT (drawer->properties),
+						    "pixmap");
+	GtkWidget    *tipentry = g_object_get_data (G_OBJECT (drawer->properties),
+						    "tooltip");
 	char         *s;
+	const char   *cs;
 
 	g_free (drawer->pixmap);
 	drawer->pixmap = NULL;
 	g_free (drawer->tooltip);
 	drawer->tooltip = NULL;
-	s = hack_icon_entry_get_icon (GNOME_ICON_ENTRY (pixentry));
+	s = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (pixentry));
 	if (string_empty (s)) {
-		drawer->pixmap = gnome_pixmap_file ("panel-drawer.png");
-		button_widget_set_pixmap (BUTTON_WIDGET(drawer->button),
-					  drawer->pixmap, -1);
+		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+							    "panel-drawer.png", TRUE, NULL);
+		button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
+					  drawer->pixmap,-1);
 	} else {
-		if(button_widget_set_pixmap(BUTTON_WIDGET(drawer->button), s, -1))
-			drawer->pixmap = g_strdup(s);
+		if(button_widget_set_pixmap (BUTTON_WIDGET (drawer->button), s, -1))
+			drawer->pixmap = g_strdup (s);
 		else {
-			drawer->pixmap = gnome_pixmap_file ("panel-drawer.png");
-			button_widget_set_pixmap(BUTTON_WIDGET(drawer->button),
-						 drawer->pixmap, -1);
+			drawer->pixmap = gnome_program_locate_file (NULL, 
+								    GNOME_FILE_DOMAIN_PIXMAP, 
+								    "panel-drawer.png",
+								     TRUE, NULL);
+			button_widget_set_pixmap (BUTTON_WIDGET (drawer->button),
+						  drawer->pixmap, -1);
 		}
 	}
 	g_free(s);
-	s = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
-	if (string_empty (s))
+	cs = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(tipentry))));
+	if (string_empty (cs))
 		drawer->tooltip = NULL;
 	else
-		drawer->tooltip = g_strdup(s);
+		drawer->tooltip = g_strdup (cs);
 
 	gtk_tooltips_set_tip (panel_tooltips, drawer->button,
 			      drawer->tooltip, NULL);
@@ -85,7 +94,7 @@ properties_close_callback(GtkWidget *widget, gpointer data)
 static void
 set_toggle (GtkWidget *widget, gpointer data)
 {
-	PerPanelConfig *ppc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	PerPanelConfig *ppc = g_object_get_data (G_OBJECT (widget), "PerPanelConfig");
 	int *the_toggle = data;
 
 	*the_toggle = GTK_TOGGLE_BUTTON(widget)->active;
@@ -122,9 +131,6 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 	w = make_size_widget (ppc);
 	gtk_box_pack_start (GTK_BOX (box_in), w, FALSE, FALSE, 0);
 
-	w = make_level_widget (ppc);
-	gtk_box_pack_start (GTK_BOX (box_in), w, FALSE, FALSE, 0);
-
 	f = gtk_frame_new (_("Size and Position"));
 	gtk_container_add (GTK_CONTAINER (f), box_in);
 	gtk_box_pack_start (GTK_BOX (box), f, FALSE, FALSE, 0);
@@ -137,13 +143,13 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 			      drawer->tooltip,
 			      (UpdateFunction)panel_config_register_changes,
 			      ppc);
-	gtk_object_set_data(GTK_OBJECT(dialog),"tooltip",w);
+	g_object_set_data (G_OBJECT (dialog), "tooltip", w);
 	
 	w = create_icon_entry(table, "icon", 0, 2, _("Icon"),
 			      NULL, drawer->pixmap,
 			      (UpdateFunction)panel_config_register_changes,
 			      ppc);
-	gtk_object_set_data(GTK_OBJECT(dialog), "pixmap", w);
+	g_object_set_data (G_OBJECT (dialog), "pixmap", w);
 
 	f = gtk_frame_new(_("Applet appearance"));
 	gtk_container_add(GTK_CONTAINER(f),table);
@@ -156,25 +162,25 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 	/*we store this in w for later use!, so don't use w as temp from now
 	  on*/
 	w = button = gtk_check_button_new_with_label (_("Enable hidebutton"));
-	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	if (ppc->hidebuttons)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
-			    GTK_SIGNAL_FUNC (set_toggle),
+	g_signal_connect (G_OBJECT (button), "toggled", 
+			    G_CALLBACK (set_toggle),
 			    &ppc->hidebuttons);
 	gtk_box_pack_start (GTK_BOX (box_in), button, TRUE, FALSE, 0);
 
 	button = gtk_check_button_new_with_label (_("Enable hidebutton arrow"));
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (set_sensitive_toggle),
+	g_signal_connect (G_OBJECT (w), "toggled", 
+			    G_CALLBACK (set_sensitive_toggle),
 			    button);
 	if (!ppc->hidebuttons)
 		gtk_widget_set_sensitive(button,FALSE);
-	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	if (ppc->hidebutton_pixmaps)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
-			    GTK_SIGNAL_FUNC (set_toggle),
+	g_signal_connect (G_OBJECT (button), "toggled", 
+			    G_CALLBACK (set_toggle),
 			    &ppc->hidebutton_pixmaps);
 	gtk_box_pack_start (GTK_BOX (box_in), button, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(f),box_in);
@@ -184,8 +190,8 @@ add_drawer_properties_page (PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer
 	gtk_notebook_append_page (GTK_NOTEBOOK(prop_nbook),
 				  box, gtk_label_new (_("Drawer")));
 	
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-			   (GtkSignalFunc) properties_close_callback,
+	g_signal_connect (G_OBJECT(dialog), "destroy",
+			  G_CALLBACK (properties_close_callback),
 			   drawer);
 
 	ppc->update_function = properties_apply_callback;
@@ -200,19 +206,24 @@ drawer_click(GtkWidget *w, Drawer *drawer)
 	DrawerWidget *drawerw = DRAWER_WIDGET(drawer->drawer);
 	PanelWidget *parent = PANEL_WIDGET(drawer->button->parent);
 	GtkWidget *panelw = parent->panel_parent;
-	
+#ifdef DRAWER_DEBUG
+	printf ("Registering drawer click \n");
+#endif	
 	switch (BASEP_WIDGET (drawerw)->state) {
 	case BASEP_SHOWN:
 	case BASEP_AUTO_HIDDEN:
+#ifdef DRAWER_DEBUG
+	printf ("Drawer closing\n");
+#endif
 		drawer_widget_close_drawer (drawerw, panelw);
 		break;
 	case BASEP_HIDDEN_LEFT:
 	case BASEP_HIDDEN_RIGHT:
+#ifdef DRAWER_DEBUG
+	printf ("Drawer opening\n");
+#endif
 		drawer_widget_open_drawer (drawerw, panelw);
 		break;
-	case BASEP_MOVING:
-		g_assert_not_reached ();
-		break;
 	}
 }
 
@@ -233,11 +244,13 @@ enter_notify_drawer(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 {
 	Drawer *drawer = data;
 	BasePWidget *basep = BASEP_WIDGET (drawer->drawer);
-
+#ifdef DRAWER_DEBUG
+	printf ("Enter notify drawer\n");
+#endif
 	if (!xstuff_is_compliant_wm() || global_config.autoraise)
 		gdk_window_raise(drawer->drawer->window);
 
-	if (basep->state == BASEP_MOVING)
+	if (basep->moving)
 		return FALSE;
 	
 	if ((basep->state != BASEP_AUTO_HIDDEN) ||
@@ -280,7 +293,7 @@ drag_data_get_cb (GtkWidget          *widget,
 {
 	char *foo;
 
-	foo = g_strdup_printf ("DRAWER:%d", find_applet (widget));
+	foo = g_strdup_printf ("DRAWER:%d", panel_find_applet (widget));
 
 	gtk_selection_data_set (selection_data,
 				selection_data->target, 8, (guchar *)foo,
@@ -292,7 +305,7 @@ drag_data_get_cb (GtkWidget          *widget,
 static Drawer *
 create_drawer_applet(GtkWidget * drawer_panel,
 		     const char *tooltip, const char *pixmap,
-		     PanelOrientType orient)
+		     PanelOrient orient)
 {
         static GtkTargetEntry dnd_targets[] = {
 		{ "application/x-panel-applet-internal", 0, 0 }
@@ -309,12 +322,12 @@ create_drawer_applet(GtkWidget * drawer_panel,
 		drawer->tooltip = g_strdup (tooltip);
 
 	if (string_empty (pixmap)) {
-		drawer->pixmap = gnome_pixmap_file ("panel-drawer.png");
+		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+							    "panel-drawer.png", TRUE, NULL);
 	} else {
 		drawer->pixmap = g_strdup (pixmap);
 	}
 	drawer->button = button_widget_new (drawer->pixmap, -1,
-					    DRAWER_TILE,
 					    TRUE, orient,
 					    _("Drawer"));
 
@@ -322,33 +335,33 @@ create_drawer_applet(GtkWidget * drawer_panel,
 	  widgets (which we actually kind of are) this will select
 	  some (already selected) events on the panel instead of
 	  the button window (where they are also selected) but
-	  we don't mind*/
+	  we don't mind*/ 
 	GTK_WIDGET_UNSET_FLAGS (drawer->button, GTK_NO_WINDOW);
+ 
 	gtk_drag_source_set (drawer->button,
 			     GDK_BUTTON1_MASK,
 			     dnd_targets, 1,
 			     GDK_ACTION_MOVE);
 	GTK_WIDGET_SET_FLAGS (drawer->button, GTK_NO_WINDOW);
 
-	gtk_signal_connect (GTK_OBJECT (drawer->button), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get_cb),
-			    NULL);
+	g_signal_connect (G_OBJECT (drawer->button), "drag_data_get",
+			  G_CALLBACK (drag_data_get_cb),
+			  NULL);
 
 	gtk_widget_show(drawer->button);
 
 	drawer->drawer = drawer_panel;
 
-	gtk_signal_connect (GTK_OBJECT (drawer->button), "clicked",
-			    GTK_SIGNAL_FUNC (drawer_click), drawer);
-	gtk_signal_connect (GTK_OBJECT (drawer->button), "destroy",
-			    GTK_SIGNAL_FUNC (destroy_drawer), drawer);
-	gtk_signal_connect (GTK_OBJECT (drawer->button), "enter_notify_event",
-			    GTK_SIGNAL_FUNC (enter_notify_drawer), drawer);
-	gtk_signal_connect (GTK_OBJECT (drawer->button), "leave_notify_event",
-			    GTK_SIGNAL_FUNC (leave_notify_drawer), drawer);
+	g_signal_connect (G_OBJECT (drawer->button), "clicked",
+			    G_CALLBACK (drawer_click), drawer);
+	g_signal_connect (G_OBJECT (drawer->button), "destroy",
+			    G_CALLBACK (destroy_drawer), drawer);
+	g_signal_connect (G_OBJECT (drawer->button), "enter_notify_event",
+			    G_CALLBACK (enter_notify_drawer), drawer);
+	g_signal_connect (G_OBJECT (drawer->button), "leave_notify_event",
+			    G_CALLBACK (leave_notify_drawer), drawer);
 
-	gtk_object_set_user_data (GTK_OBJECT (drawer->button), drawer);
-	gtk_object_set_data (GTK_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
+	g_object_set_data (G_OBJECT (drawer_panel), DRAWER_PANEL_KEY, drawer);
 	gtk_widget_queue_resize (GTK_WIDGET (drawer_panel));
 
 	return drawer;
@@ -356,14 +369,12 @@ create_drawer_applet(GtkWidget * drawer_panel,
 
 static Drawer *
 create_empty_drawer_applet(const char *tooltip, const char *pixmap,
-			   PanelOrientType orient)
+			   PanelOrient orient)
 {
-	GtkWidget *dw = drawer_widget_new (orient,
+	GtkWidget *dw = drawer_widget_new (NULL, orient,
 					   BASEP_EXPLICIT_HIDE,
 					   BASEP_SHOWN,
-					   BASEP_LEVEL_DEFAULT,
-					   FALSE,
-					   SIZE_STANDARD,
+					   PANEL_SIZE_MEDIUM,
 					   TRUE, TRUE,
 					   PANEL_BACK_NONE, NULL,
 					   TRUE, FALSE, TRUE, NULL);
@@ -371,12 +382,12 @@ create_empty_drawer_applet(const char *tooltip, const char *pixmap,
 }
 
 void
-set_drawer_applet_orient(Drawer *drawer, PanelOrientType orient)
+set_drawer_applet_orient(Drawer *drawer, PanelOrient orient)
 {
-	g_return_if_fail(drawer!=NULL);
+	g_return_if_fail (drawer != NULL);
 
-	button_widget_set_params(BUTTON_WIDGET(drawer->button),
-				 DRAWER_TILE,TRUE,orient);
+	button_widget_set_params (BUTTON_WIDGET (drawer->button),
+				  TRUE, orient);
 	
 	/*ignore orient events until we are realized, this will only
 	  be the initial one and we have already set the orientation*/
@@ -393,8 +404,8 @@ drawer_setup(Drawer *drawer)
 	if(BASEP_WIDGET(drawer->drawer)->state != BASEP_SHOWN) {
 		GtkRequisition chreq;
 		gtk_widget_size_request(drawer->drawer, &chreq);
-		gtk_widget_set_uposition(drawer->drawer,
-					 -chreq.width - 1, -chreq.height - 1);
+		gtk_window_move (GTK_WINDOW (drawer->drawer),
+				 -chreq.width - 1, -chreq.height - 1);
 	}
 	gtk_widget_show(drawer->drawer);
 }
@@ -405,19 +416,25 @@ button_size_alloc(GtkWidget *widget, GtkAllocation *alloc, Drawer *drawer)
 	if(!GTK_WIDGET_REALIZED(widget))
 		return;
 
-	gtk_widget_queue_resize(drawer->drawer);
+	gtk_widget_queue_resize (drawer->drawer);
 
-	gtk_object_set_data(GTK_OBJECT(widget),"allocated",GINT_TO_POINTER(1));
+	g_object_set_data (G_OBJECT (widget), "allocated", GINT_TO_POINTER (TRUE));
 }
 
-gboolean
-load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
-		    PanelWidget *panel, int pos, gboolean exactpos)
+Drawer *
+load_drawer_applet (gchar       *mypanel_id,
+		    const char  *pixmap,
+		    const char  *tooltip,
+		    PanelWidget *panel,
+		    int          pos,
+		    gboolean     exactpos,
+		    const char  *gconf_key)
 {
-	Drawer *drawer;
-	PanelOrientType orient = get_applet_orient (panel);
+	Drawer      *drawer;
+	PanelOrient  orient = get_applet_orient (panel);
+	AppletInfo  *info;
 
-	if (mypanel_id < 0) {
+	if (!mypanel_id) {
 		drawer = create_empty_drawer_applet (tooltip, pixmap, orient);
 		if (drawer != NULL)
 			panel_setup (drawer->drawer);
@@ -432,7 +449,7 @@ load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
 			drawer = create_empty_drawer_applet(tooltip, pixmap, orient);
 			if(drawer) panel_setup(drawer->drawer);
 			panels_to_sync = TRUE;
-		} else if ( ! IS_DRAWER_WIDGET (dr_pd->panel)) {
+		} else if ( ! DRAWER_IS_WIDGET (dr_pd->panel)) {
 			g_warning ("I found a bogus panel for a drawer, making a new one");
 			drawer = create_empty_drawer_applet(tooltip, pixmap, orient);
 			if(drawer) panel_setup(drawer->drawer);
@@ -446,30 +463,35 @@ load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
 		}
 	}
 
-	if(!drawer)
-		return FALSE;
+	if (!drawer)
+		return NULL;
 
 	{
-		GtkWidget *dw = drawer->drawer;
+		GtkWidget  *dw = drawer->drawer;
 
-		if(!register_toy(drawer->button,
-				 drawer, (GDestroyNotify)g_free,
-				 panel, pos, exactpos, APPLET_DRAWER)) {
-			/* by this time drawer has been freed as register_toy
-			   has destroyed drawer->button */
-			gtk_widget_destroy(dw);
-			return FALSE;
+		info = panel_applet_register (drawer->button,
+					      drawer,
+					      (GDestroyNotify) g_free,
+					      panel,
+					      pos,
+					      exactpos,
+					      APPLET_DRAWER,
+					      gconf_key);
+
+		if (!info) {
+			gtk_widget_destroy (dw);
+			return NULL;
 		}
 	}
 
-	gtk_signal_connect_after(GTK_OBJECT(drawer->button),
-				 "size_allocate",
-				 GTK_SIGNAL_FUNC(button_size_alloc),
-				 drawer);
+	g_signal_connect_after (G_OBJECT(drawer->button),
+				"size_allocate",
+				G_CALLBACK (button_size_alloc),
+				drawer);
 
 	/* this doesn't make sense anymore */
 	if((BASEP_WIDGET(drawer->drawer)->state == BASEP_SHOWN) &&
-	   (IS_BASEP_WIDGET (panel->panel_parent))) {
+	   (BASEP_IS_WIDGET (panel->panel_parent))) {
 		/*pop up, if popped down, if it's not an autohidden
 		  widget then it will just ignore this next call */
 		basep_widget_autoshow(BASEP_WIDGET(panel->panel_parent));
@@ -477,19 +499,110 @@ load_drawer_applet (int mypanel_id, const char *pixmap, const char *tooltip,
 
 	panel_widget_add_forbidden(PANEL_WIDGET(BASEP_WIDGET(drawer->drawer)->panel));
 
-	gtk_tooltips_set_tip (panel_tooltips, drawer->button,
-			      drawer->tooltip, NULL);
+	gtk_tooltips_set_tip (panel_tooltips,drawer->button,
+			      drawer->tooltip,NULL);
 	drawer_setup (drawer);
 
-	g_assert (applets_last != NULL);
+	if (!commie_mode)
+		panel_applet_add_callback (info,
+					   "properties",
+					   GTK_STOCK_PROPERTIES,
+					   _("Properties..."));
 
-	if ( ! commie_mode)
-		applet_add_callback(applets_last->data,"properties",
-				    GNOME_STOCK_MENU_PROP,
-				    _("Properties..."));
-	applet_add_callback(applets_last->data, "help",
-			    GNOME_STOCK_PIXMAP_HELP,
-			    _("Help"));
-	return TRUE;
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("Help"));
+
+	return drawer;
 }
 
+void
+drawer_save_to_gconf (Drawer     *drawer,
+		      const char *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+
+	g_return_if_fail (drawer && BASEP_IS_WIDGET (drawer->drawer));
+	g_return_if_fail (PANEL_IS_WIDGET (BASEP_WIDGET (drawer->drawer)->panel));
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "parameters");
+	gconf_client_set_int (client, temp_key,
+			      g_slist_index (panels, BASEP_WIDGET (drawer->drawer)->panel),
+			      NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "unique-drawer-panel-id");
+	gconf_client_set_string (client, temp_key,
+				 PANEL_WIDGET (BASEP_WIDGET (drawer->drawer)->panel)->unique_id,
+				 NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "pixmap");
+	gconf_client_set_string (client, temp_key, drawer->pixmap, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "tooltip");
+	gconf_client_set_string (client, temp_key, drawer->tooltip, NULL);
+	g_free (temp_key);
+
+	g_free (profile);
+}
+
+void
+drawer_load_from_gconf (PanelWidget *panel_widget,
+			gint         position,
+			const char  *gconf_key,
+			gboolean     use_default)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+	int          panel;
+	char        *panel_id;
+	char        *pixmap;
+	char        *tooltip;
+
+	g_return_if_fail (panel_widget);
+	g_return_if_fail (gconf_key);
+
+	client  = panel_gconf_get_client ();
+
+	/* FIXME: Screen widths etc.. */
+	if (use_default)
+		profile = g_strdup ("medium");
+	else
+		profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "parameters", use_default);
+	panel = gconf_client_get_int (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "unique-drawer-panel-id", use_default);
+	panel_id = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "pixmap", use_default);
+	pixmap = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	temp_key = panel_gconf_objects_get_full_key (profile, gconf_key, "tooltip", use_default);
+	tooltip = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	if (!panel_id < 0 && panel >= 0) {
+		PanelWidget *pw = g_slist_nth_data (panels, panel);
+
+		if (pw)
+			panel_id = g_strdup (pw->unique_id);
+	}
+
+	load_drawer_applet (panel_id, pixmap, tooltip, panel_widget, position, TRUE, gconf_key);
+
+	g_free (panel_id);
+	g_free (pixmap);
+	g_free (tooltip);
+	g_free (profile);
+}
