@@ -8,16 +8,14 @@
 #ifndef FOOBAR_WIDGET_H
 #define FOOBAR_WIDGET_H
 
-#include <gnome.h>
 #include "panel-types.h"
-#include "gwmh.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
-#define TYPE_FOOBAR_WIDGET     (foobar_widget_get_type ())
-#define FOOBAR_WIDGET(o)       (GTK_CHECK_CAST ((o), TYPE_FOOBAR_WIDGET, FoobarWidget))
-#define FOOBAR_WIDGET_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), TYPE_FOOBAR_WIDGET, FoobarWidgetType))
-#define IS_FOOBAR_WIDGET(o)    (GTK_CHECK_TYPE ((o), TYPE_FOOBAR_WIDGET))
+#define FOOBAR_TYPE_WIDGET     		(foobar_widget_get_type ())
+#define FOOBAR_WIDGET(object)       	(G_TYPE_CHECK_INSTANCE_CAST ((object), FOOBAR_TYPE_WIDGET, FoobarWidget))
+#define FOOBAR_WIDGET_CLASS(klass) 	(G_TYPE_CHECK_CLASS_CAST ((klass), FOOBAR_TYPE_WIDGET, FoobarWidgetType))
+#define FOOBAR_IS_WIDGET(object)    	(G_TYPE_CHECK_INSTANCE_TYPE ((object), FOOBAR_TYPE_WIDGET))
 
 typedef struct _FoobarWidget           FoobarWidget;
 typedef struct _FoobarWidgetClass      FoobarWidgetClass;
@@ -38,18 +36,16 @@ struct _FoobarWidget
 	guint clock_timeout;
 
 	GtkWidget *programs;
-	GtkWidget *favorites;
-	GtkWidget *settings;
 
-	GHashTable *tasks;
+	GHashTable *windows;
 	GtkWidget *task_item;
 	GtkWidget *task_menu;
-	GtkWidget *task_pixmap;
+	GtkWidget *task_image;
 	GtkWidget *task_bin;
-	GwmhTask *icon_task; /* the task whoose icon is shown,
-			      * hopefully should be always OK,
-			      * but we only use the pointer value
-			      * and never dereference this */
+	gpointer icon_window; /* the task whoose icon is shown,
+			       * hopefully should be always OK,
+			       * but we only use the pointer value
+			       * and never dereference this */
 
 	gboolean compliant_wm;
 	char *clock_format;
@@ -62,8 +58,9 @@ struct _FoobarWidgetClass
 	GtkWindowClass panel_class;
 };
 
-GtkType		foobar_widget_get_type		(void) G_GNUC_CONST;
-GtkWidget *	foobar_widget_new		(void);
+GType		foobar_widget_get_type		(void) G_GNUC_CONST;
+
+GtkWidget *	foobar_widget_new		(const char *panel_id, int screen);
 
 void		foobar_widget_update_winhints	(FoobarWidget *foo);
 void		foobar_widget_redo_window	(FoobarWidget *foo);
@@ -79,6 +76,6 @@ void		foobar_widget_force_menu_remake	(void);
 
 void		foobar_widget_global_set_clock_format (const char *format);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
