@@ -29,7 +29,8 @@
 #include "menu-fentry.h"
 #include "menu.h"
 #include "panel-util.h"
-#include "panel_config_global.h"
+#include "panel-config-global.h"
+#include "panel-gconf.h"
 #include "session.h"
 
 #include "gnome-desktop-item.h"
@@ -50,11 +51,7 @@ extern GlobalConfig global_config;
 extern gboolean commie_mode;
 
 enum {
-	HELP_BUTTON,
-	OK_BUTTON,
-	CANCEL_BUTTON,
-	REVERT_BUTTON,
-	CLOSE_BUTTON
+	REVERT_BUTTON
 };
 
 static void
@@ -89,7 +86,7 @@ launch_url (Launcher *launcher)
 
 
 static void
-launch (Launcher *launcher, int argc, char *argv[])
+launch (Launcher *launcher)
 {
 	GnomeDesktopItem *item;
 	GnomeDesktopItemType type;
@@ -104,10 +101,14 @@ launch (Launcher *launcher, int argc, char *argv[])
 		launch_url (launcher);
 	} else {
 		GError *error = NULL;
-		gnome_desktop_item_launch (item, argc, argv, &error);
+		gnome_desktop_item_launch (item,
+					   NULL /* file_list */,
+					   0 /* flags */,
+					   &error);
 		if (error != NULL) {
 			panel_error_dialog ("cannot_launch_icon",
-					    _("Cannot launch icon\n%s"),
+					    _("<b>Cannot launch icon</b>\n\n"
+					      "Details: %s"),
 					    error->message);
 			g_clear_error (&error);
 		}
@@ -131,7 +132,7 @@ launch (Launcher *launcher, int argc, char *argv[])
 static void
 launch_cb (GtkWidget *widget, gpointer data)
 {
-	launch (data, 0, NULL);
+	launch (data);
 }
 
 static void
@@ -144,12 +145,12 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       Launcher         *launcher)
 {
-	GList *list;
 	GError *error = NULL;
 
-	list = gnome_vfs_uri_list_parse ((char *)selection_data->data);
-
-	gnome_desktop_item_drop_uri_list (launcher->ditem, list, &error);
+	gnome_desktop_item_drop_uri_list (launcher->ditem,
+					  (const char *)selection_data->data,
+					  0 /* flags */,
+					  &error);
 	if (error != NULL) {
 		panel_error_dialog ("cannot_launch_icon",
 				    _("Cannot launch icon\n%s"),
@@ -157,8 +158,6 @@ drag_data_received_cb (GtkWidget        *widget,
 		g_clear_error (&error);
 	}
 
-	gnome_vfs_uri_list_free (list);
-
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
@@ -324,15 +323,16 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	if (ditem == NULL) {
 		if (parameters == NULL) {
 			return NULL;
-		} else if (strstr (parameters, "file://")) {
-
+		}
 #ifdef LAUNCHER_DEBUG
-	printf ("Creating gnome_desktop_item_new_from_file\n");
+	printf ("Creating gnome_desktop_item_new_from_uri\n");
 #endif
 		ditem = gnome_desktop_item_new_from_uri (parameters,
 							 0 /* flags */,
 							 NULL /* error */);
-		} else {
+
+		
+		if (ditem == NULL) {
 			gchar *entry;
 
 			entry = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR, 
@@ -359,7 +359,6 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	/* Icon/Text will be setup later */
 	launcher->button = button_widget_new (NULL /* icon */,
 					      -1,
-					      LAUNCHER_POBJECT,
 					      FALSE,
 					      PANEL_ORIENT_UP,
 					      NULL /* text */);
@@ -410,8 +409,6 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 			    G_CALLBACK(destroy_launcher),
 			    launcher);
 
-	gtk_object_set_user_data(GTK_OBJECT(launcher->button), launcher);
-
 	launcher->ditem = ditem;
 
 	return launcher;
@@ -423,17 +420,15 @@ setup_button (Launcher *launcher)
 	const char *comment;
 	const char *name;
 	const char *docpath;
-	char *real_docpath;
 	char *str;
 	char *icon;
+	
+	g_return_if_fail (launcher != NULL);
 
-	name = gnome_desktop_item_get_string (launcher->ditem,
-					      GNOME_DESKTOP_ITEM_NAME);
-	comment = gnome_desktop_item_get_string (launcher->ditem,
-						 GNOME_DESKTOP_ITEM_COMMENT);
-
-	/* GnomeDesktopItem makes this guarantee */
-	g_assert (name != NULL);
+	name = gnome_desktop_item_get_localestring (launcher->ditem,
+						    GNOME_DESKTOP_ITEM_NAME);
+	comment = gnome_desktop_item_get_localestring (launcher->ditem,
+						       GNOME_DESKTOP_ITEM_COMMENT);
 
 	/* Setup tooltip */
 	if (comment != NULL)
@@ -461,18 +456,15 @@ setup_button (Launcher *launcher)
 
 	panel_applet_remove_callback (launcher->info, "help_on_app");
 
-	real_docpath = panel_gnome_kde_help_path (docpath);
-	if (real_docpath != NULL) {
+	if (docpath != NULL) {
 		char *title;
 
-		g_free (real_docpath);
-
 		title = g_strdup_printf (_("Help on %s Application"), name);
 
 		panel_applet_add_callback (launcher->info,
 					   "help_on_app",
-					    GTK_STOCK_HELP,
-					    title);
+					   GTK_STOCK_HELP,
+					   title);
 		g_free (title);
 	}
 }
@@ -483,7 +475,7 @@ properties_apply (Launcher *launcher)
 {
 	char *location;
 
-	/* save (steal) location */
+	/* save location */
 	location = g_strdup (gnome_desktop_item_get_location (launcher->ditem));
 
 	gnome_desktop_item_unref (launcher->ditem);
@@ -520,7 +512,7 @@ window_response (GtkWidget *w, int response, gpointer data)
 {
 	Launcher *launcher = data;
 
-	if (response == HELP_BUTTON) {
+	if (response == GTK_RESPONSE_HELP) {
 		panel_show_help ("launchers", NULL);
 	} else if (response == REVERT_BUTTON) { /* revert */
 		gnome_ditem_edit_set_ditem (GNOME_DITEM_EDIT (launcher->dedit),
@@ -547,12 +539,12 @@ create_properties_dialog (Launcher *launcher)
 	dialog = gtk_dialog_new_with_buttons (_("Launcher properties"),
 					      NULL /* parent */,
 					      0 /* flags */,
-					      GTK_STOCK_CLOSE,
-					      CLOSE_BUTTON,
+					      GTK_STOCK_HELP,
+					      GTK_RESPONSE_HELP,
 					      GTK_STOCK_REVERT_TO_SAVED,
 					      REVERT_BUTTON,
-					      GTK_STOCK_HELP,
-					      HELP_BUTTON,
+					      GTK_STOCK_CLOSE,
+					      GTK_RESPONSE_CLOSE,
 					      NULL);
 
 	launcher->dedit = gnome_ditem_edit_new ();
@@ -561,7 +553,6 @@ create_properties_dialog (Launcher *launcher)
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 			       "launcher_properties", "Panel");
-	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, TRUE);
 	
 	if (launcher->revert_ditem != NULL)
 		gnome_desktop_item_unref (launcher->revert_ditem);
@@ -570,25 +561,6 @@ create_properties_dialog (Launcher *launcher)
 	gnome_ditem_edit_set_ditem (GNOME_DITEM_EDIT (launcher->dedit),
 				    launcher->ditem);
 
-	/* FIXME: this is 1) ugly, 2) needs to be done in a better way,
-	 * FIXME: read the dialog proposal */
-
-#if 0
-#define SETUP_EDITABLE(entry_name)					\
-	gnome_dialog_editable_enters					\
-		(GNOME_DIALOG (dialog),					\
-		 GTK_EDITABLE (gnome_ditem_get_##entry_name##_entry  	\
-			       (GNOME_DITEM_EDIT (launcher->dedit))));
-
-	SETUP_EDITABLE (name);
-	SETUP_EDITABLE (comment);
-	SETUP_EDITABLE (exec);
-	SETUP_EDITABLE (tryexec);
-	SETUP_EDITABLE (doc);
-
-#undef SETUP_EDITABLE
-#endif
-	
 	g_signal_connect (G_OBJECT (launcher->dedit), "changed",
 			    G_CALLBACK (launcher_changed),
 			    launcher);
@@ -597,6 +569,7 @@ create_properties_dialog (Launcher *launcher)
 			    G_CALLBACK (properties_close_callback),
 			    launcher);
 
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
 	g_signal_connect (G_OBJECT (dialog), "response",
 			    G_CALLBACK (window_response),
@@ -611,19 +584,65 @@ void
 launcher_properties (Launcher *launcher)
 {
 	if (launcher->prop_dialog != NULL) {
-		gtk_widget_show_now (launcher->prop_dialog);
-		gdk_window_raise (launcher->prop_dialog->window);
+		gtk_window_present (GTK_WINDOW (launcher->prop_dialog));
 		return;
 	}
 
 	launcher->prop_dialog = create_properties_dialog (launcher);
 	gtk_widget_show_all (launcher->prop_dialog);
-	panel_set_dialog_layer (launcher->prop_dialog);
+}
+
+void
+launcher_save_to_gconf (Launcher  *launcher,
+			const char *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+	const char  *location;
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	location = gnome_desktop_item_get_location (launcher->ditem);
+
+	temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "launcher-location");
+	gconf_client_set_string (client, temp_key, location, NULL);
+	g_free (temp_key);
+}
+
+void
+launcher_load_from_gconf (PanelWidget *panel_widget,
+			  gint         position,
+			  const char  *gconf_key,
+			  gboolean     use_default)
+{
+        GConfClient *client;
+        char        *temp_key;
+        char        *launcher_location;
+
+        g_return_if_fail (panel_widget);
+        g_return_if_fail (gconf_key);
+
+        client  = panel_gconf_get_client ();
+	/* FIXME : We need to do checking of screen dimensions for default */
+        temp_key = use_default ? panel_gconf_objects_default_profile_get_full_key ("medium", gconf_key, "launcher-location") :
+		panel_gconf_objects_profile_get_full_key (session_get_current_profile (), gconf_key, "launcher-location");
+        launcher_location = gconf_client_get_string (client, temp_key, NULL);
+        g_free (temp_key);
+
+	load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
+
+        g_free (launcher_location);
 }
 
 Launcher *
-load_launcher_applet_full (const char *params, GnomeDesktopItem *ditem,
-			   PanelWidget *panel, int pos, gboolean exactpos)
+load_launcher_applet_full (const char       *params,
+			   GnomeDesktopItem *ditem,
+			   PanelWidget      *panel,
+			   int               pos,
+			   gboolean          exactpos,
+			   const char       *gconf_key)
 {
 	Launcher   *launcher;
 
@@ -634,7 +653,8 @@ load_launcher_applet_full (const char *params, GnomeDesktopItem *ditem,
 
 	launcher->info = panel_applet_register (launcher->button, launcher,
 						free_launcher, panel, pos, 
-						exactpos, APPLET_LAUNCHER);
+						exactpos, APPLET_LAUNCHER,
+						gconf_key);
 	if (!launcher->info) {
 		/* 
 		 * Don't free launcher here, the button has 
@@ -662,23 +682,23 @@ static void
 really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 {
 	GnomeDItemEdit *dedit = GNOME_DITEM_EDIT(data);
-	int pos = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(dialog),"pos"));
-	gboolean exactpos = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(dialog),"exactpos"));
-	PanelWidget *panel = gtk_object_get_data(GTK_OBJECT(dialog),"panel");
+	int pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "pos"));
+	gboolean exactpos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "exactpos"));
+	PanelWidget *panel = g_object_get_data (G_OBJECT (dialog), "panel");
 	GnomeDesktopItem *ditem;
 	
-	if (response == OK_BUTTON) {
+	if (response == GTK_RESPONSE_OK) {
 		Launcher *launcher;
 
 		ditem = gnome_ditem_edit_get_ditem (dedit);
 		ditem = gnome_desktop_item_copy (ditem);
 
-		launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos);
+		launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos, NULL);
 		if (launcher != NULL)
 			launcher_hoard (launcher);
 
 		panel_config_sync_schedule ();
-	} else if (response == HELP_BUTTON) {
+	} else if (response == GTK_RESPONSE_HELP) {
 		panel_show_help ("launchers", "LAUNCHERS");
 		/* just return as we don't want to close */
 		return;
@@ -697,41 +717,22 @@ ask_about_launcher (const char *file, PanelWidget *panel, int pos, gboolean exac
 	dialog = gtk_dialog_new_with_buttons (_("Create launcher applet"),
 					      NULL /* parent */,
 					      0 /* flags */,
-					      GTK_STOCK_CANCEL,
-					      CANCEL_BUTTON,
-					      GTK_STOCK_OK,
-					      OK_BUTTON,
 					      GTK_STOCK_HELP,
-					      HELP_BUTTON,
+					      GTK_RESPONSE_HELP,
+					      GTK_STOCK_CANCEL,
+					      GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OK,
+					      GTK_RESPONSE_OK,
 					      NULL);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"create_launcher", "Panel");
-	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, TRUE);
 	
 	dee = GNOME_DITEM_EDIT (gnome_ditem_edit_new ());
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    GTK_WIDGET (dee),
 			    TRUE, TRUE, GNOME_PAD_SMALL);
 
-	/* FIXME: this is 1) ugly, 2) needs to be done in a better way,
-	 * FIXME: read the dialog proposal */
-
-#if 0
-#define SETUP_EDITABLE(entry_name)					\
-	gnome_dialog_editable_enters					\
-		(GNOME_DIALOG (dialog),					\
-		 GTK_EDITABLE (gnome_ditem_get_##entry_name##_entry (dee)));
-
-	SETUP_EDITABLE (name);
-	SETUP_EDITABLE (comment);
-	SETUP_EDITABLE (exec);
-	SETUP_EDITABLE (tryexec);
-	SETUP_EDITABLE (doc);
-
-#undef SETUP_EDITABLE
-#endif
-
 	ditem = gnome_desktop_item_new ();
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, file);
 	gnome_desktop_item_set_entry_type (ditem, GNOME_DESKTOP_ITEM_TYPE_APPLICATION);
@@ -740,19 +741,18 @@ ask_about_launcher (const char *file, PanelWidget *panel, int pos, gboolean exac
 
 	gnome_desktop_item_unref (ditem);
 	
-	gtk_object_set_data (GTK_OBJECT(dialog), "pos", GINT_TO_POINTER (pos));
-	gtk_object_set_data (GTK_OBJECT(dialog), "exactpos",
-			     GINT_TO_POINTER (exactpos));
-	gtk_object_set_data (GTK_OBJECT (dialog), "panel", panel);
+	g_object_set_data (G_OBJECT(dialog), "pos", GINT_TO_POINTER (pos));
+	g_object_set_data (G_OBJECT(dialog), "exactpos",
+			   GINT_TO_POINTER (exactpos));
+	g_object_set_data (G_OBJECT (dialog), "panel", panel);
 
 	g_signal_connect (G_OBJECT (dialog), "response",
 			    G_CALLBACK (really_add_launcher),
 			    dee);
 
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), OK_BUTTON);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
 	gtk_widget_show_all (dialog);
-	panel_set_dialog_layer (dialog);
 
 	gnome_ditem_edit_grab_focus (dee);
 }
@@ -762,7 +762,9 @@ ditem_set_icon (GnomeDesktopItem *ditem, const char *icon)
 {
 	if (icon != NULL &&
 	    icon[0] != G_DIR_SEPARATOR) {
-		char *full = quick_desktop_item_find_icon (icon);
+		char *full = gnome_desktop_item_find_icon (icon,
+							   20 /* desired size */,
+							   0 /* flags */);
 		if (full != NULL) {
 			gnome_desktop_item_set_string (ditem,
 						       GNOME_DESKTOP_ITEM_ICON,
@@ -790,14 +792,19 @@ load_launcher_applet_from_info (const char *name, const char *comment,
 	Launcher *launcher;
 
 	ditem = gnome_desktop_item_new ();
+	/* set current language and the "C" locale to this name,
+	 * this is kind of evil... */
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_NAME, name);
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
+	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_NAME, name);
+	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
+
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, exec);
 	ditem_set_icon (ditem, icon);
 	gnome_desktop_item_set_entry_type (ditem,
 					   GNOME_DESKTOP_ITEM_TYPE_APPLICATION);
 
-	launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos);
+	launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos, NULL);
 	if (launcher != NULL)
 		launcher_save (launcher);
 
@@ -816,15 +823,19 @@ load_launcher_applet_from_info_url (const char *name, const char *comment,
 	Launcher *launcher;
 
 	ditem = gnome_desktop_item_new ();
+	/* set current language and the "C" locale to this name,
+	 * this is kind of evil... */
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_NAME, name);
-	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_COMMENT,
-				       comment);
+	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
+	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_NAME, name);
+	gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT, comment);
+
 	ditem_set_icon (ditem, icon);
 	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_URL, url);
 	gnome_desktop_item_set_entry_type (ditem,
 					   GNOME_DESKTOP_ITEM_TYPE_LINK);
 
-	launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos);
+	launcher = load_launcher_applet_full (NULL, ditem, panel, pos, exactpos, NULL);
 	if (launcher != NULL)
 		launcher_save (launcher);
 
@@ -834,16 +845,20 @@ load_launcher_applet_from_info_url (const char *name, const char *comment,
 }
 
 Launcher *
-load_launcher_applet (const char *params, PanelWidget *panel, int pos,
-		      gboolean exactpos)
+load_launcher_applet (const char  *params,
+		      PanelWidget *panel,
+		      int          pos,
+		      gboolean     exactpos,
+		      const char  *gconf_key)
 {
-	return load_launcher_applet_full (params, NULL, panel, pos, exactpos);
+	return load_launcher_applet_full (params, NULL, panel, pos, exactpos, gconf_key);
 }
 
 static char *
-launcher_get_unique_file (void)
+launcher_get_unique_uri (void)
 {
 	int rnd, word;
+	char *uri;
 #define NUM_OF_WORDS 12
 	char *words[] = {
 		"foo",
@@ -870,8 +885,11 @@ launcher_get_unique_file (void)
 		full = launcher_file_name (fname);
 		g_free (fname);
 
-		if ( ! g_file_test (full, G_FILE_TEST_EXISTS))
-			return full;
+		if ( ! g_file_test (full, G_FILE_TEST_EXISTS)) {
+			uri = gnome_vfs_get_uri_from_local_path (full);
+			g_free (full);
+			return uri;
+		}
 	}
 
 	g_assert_not_reached ();
@@ -879,43 +897,50 @@ launcher_get_unique_file (void)
 }
 
 char *
-launcher_file_name (const char *base)
+launcher_file_name (const char *location)
 {
-#ifdef PER_SESSION_CONFIGURATION
-	/* FIXME: this needs to do stuff, or perhaps just ignore per session
-	 * configs */
-#ifdef __GNUC__
-#warning FIXME: per session config must be done for launchers
-#endif
-#else
 	char *tmp, *retval;
 
-	g_return_val_if_fail (base != NULL, NULL);
+	g_return_val_if_fail (location!= NULL, NULL);
 
 	tmp = gnome_util_home_file (PANEL_CONFIG_PATH "launchers");
-	retval = g_build_filename (tmp, base, NULL);
+	/* Make sure the launcher directory exists */
+	if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
+		panel_ensure_dir (tmp);
+	}
+
+	retval = g_build_filename (tmp, location, NULL);
 
 	g_free (tmp);
 	return retval;
-#endif
 }
 
 
 void
 launcher_save (Launcher *launcher)
 {
+	GError *error;
+
 	g_return_if_fail (launcher != NULL);
 	g_return_if_fail (launcher->ditem != NULL);
 
 	if (gnome_desktop_item_get_location (launcher->ditem) == NULL)
 		gnome_desktop_item_set_location (launcher->ditem,
-						 launcher_get_unique_file ());
+						 launcher_get_unique_uri ());
 
+	error = NULL;
 	gnome_desktop_item_save (launcher->ditem,
 				 NULL /* under */,
 				 TRUE /* force */,
-				 NULL /* FIXME: error */);
-	/* FIXME: handle errors */
+				 &error);
+	if (error != NULL) {
+		panel_error_dialog ("cannot_save_launcher" /* class */,
+				    _("Cannot save launcher to disk, "
+				      "the following error occured:\n\n"
+				      "%s"),
+				    error->message);
+		g_clear_error (&error);
+	}
 }
 
 void
