@@ -11,16 +11,19 @@
 #ifndef LAUNCHER_H
 #define LAUNCHER_H
 
-#include <panel-widget.h>
+#include "applet.h"
+#include "panel-widget.h"
 
-BEGIN_GNOME_DECLS
+#include "gnome-desktop-item.h"
+
+G_BEGIN_DECLS
 
 typedef struct {
 	AppletInfo        *info;
 	GtkWidget         *button;
-	GtkObject	  *dedit;
-	GnomeDesktopEntry *dentry;
-	GnomeDesktopEntry *revert_dentry;
+	GtkWidget	  *dedit;
+	GnomeDesktopItem *ditem;
+	GnomeDesktopItem *revert_ditem;
 
 	GtkWidget         *prop_dialog;
 } Launcher;
@@ -30,19 +33,22 @@ typedef struct {
  * otherwise things may be removed from disk when they shouldn't
  * etc.  Also always hoard if an applet which owns a desktop already
  * exists.*/
-Launcher *	load_launcher_applet_full	(const char *params,
-						 GnomeDesktopEntry *dentry,
-						 PanelWidget *panel,
-						 int pos,
-						 gboolean exactpos);
-Launcher *	load_launcher_applet		(const char *params,
-						 PanelWidget *panel,
-						 int pos,
-						 gboolean exactpos);
+Launcher *	load_launcher_applet_full	(const char       *params,
+						 GnomeDesktopItem *ditem,
+						 PanelWidget      *panel,
+						 int               pos,
+						 gboolean          exactpos,
+						 const char       *gconf_key);
+
+Launcher *	load_launcher_applet		(const char       *params,
+						 PanelWidget      *panel,
+						 int              pos,
+						 gboolean         exactpos,
+						 const char      *gconf_key);
+
 Launcher *	load_launcher_applet_from_info	(const char *name,
 						 const char *comment,
-						 char **exec,
-						 int execn,
+						 const char *exec,
 						 const char *icon,
 						 PanelWidget *panel,
 						 int pos,
@@ -56,6 +62,14 @@ Launcher *	load_launcher_applet_from_info_url(const char *name,
 						   gboolean exactpos);
 void		launcher_properties		(Launcher *launcher);
 
+void            launcher_save_to_gconf          (Launcher   *launcher,
+						 const char *gconf_key);
+
+void            launcher_load_from_gconf        (PanelWidget *panel_widget,
+						 gint         position,
+						 const char  *gconf_key,
+						 gboolean     use_default);
+
 void		ask_about_launcher		(const char *file,
 						 PanelWidget *panel,
 						 int pos,
@@ -72,6 +86,6 @@ Launcher *	find_launcher			(const char *path);
 /* make file name from base */
 char *		launcher_file_name		(const char *base);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
