@@ -1,36 +1,103 @@
 /*
  * panel-applet-frame.c:
  *
- * Authors:
- *    Mark McLoughlin <mark@skynet.ie>
+ * Copyright (C) 2001 Sun Microsystems, Inc.
  *
- * Copyright 2001 Sun Microsystems, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors:
+ *	Mark McLoughlin <mark@skynet.ie>
  */
 
+#include <config.h>
+#include <string.h>
+
 #include <libbonoboui.h>
+#include <gconf/gconf.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 
 #include "panel-applet-frame.h"
+#include "panel-gconf.h"
+#include "session.h"
 #include "applet.h"
 
 #undef PANEL_APPLET_FRAME_DEBUG
 
 struct _PanelAppletFramePrivate {
-	GNOME_PanelAppletShell  applet_shell;
-	Bonobo_PropertyBag      property_bag;
+	GNOME_Vertigo_PanelAppletShell  applet_shell;
+	Bonobo_PropertyBag              property_bag;
 
-	AppletInfo             *applet_info;
+	AppletInfo                     *applet_info;
+
+	gchar                          *iid;
 };
 
 static GObjectClass *parent_class;
 
+void
+panel_applet_frame_save_to_gconf (PanelAppletFrame *frame,
+				  const char       *gconf_key)
+{
+	GConfClient *client;
+	char        *profile;
+	char        *temp_key;
+
+	client  = panel_gconf_get_client ();
+	profile = session_get_current_profile ();
+
+	temp_key = panel_gconf_applets_profile_get_full_key (profile, gconf_key, "bonobo-iid");
+	gconf_client_set_string (client, temp_key, frame->priv->iid, NULL);
+	g_free (temp_key);
+	g_free (profile);
+}
+
+void
+panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
+				    gint         position,
+				    const char  *gconf_key,
+				    gboolean     use_default)
+{
+	GConfClient *client;
+	char        *temp_key;
+	char        *applet_iid;
+
+	g_return_if_fail (panel_widget);
+	g_return_if_fail (gconf_key);
+
+	client  = panel_gconf_get_client ();
+
+	temp_key = use_default ? panel_gconf_applets_default_profile_get_full_key ("medium", gconf_key, "bonobo-iid") :
+		panel_gconf_applets_profile_get_full_key (session_get_current_profile (), gconf_key, "bonobo-iid");
+	applet_iid = gconf_client_get_string (client, temp_key, NULL);
+	g_free (temp_key);
+
+	panel_applet_frame_load (applet_iid, panel_widget, position, gconf_key);
+
+	g_free (applet_iid);
+}
+
 static void
 popup_handle_remove (BonoboUIComponent *uic,
 		     PanelAppletFrame  *frame,
 		     const gchar       *verbname)
 {
-	g_return_if_fail (frame && PANEL_IS_APPLET_FRAME (frame));
-
-	panel_applet_clean (frame->priv->applet_info);
+	AppletInfo *info = frame->priv->applet_info;
+	frame->priv->applet_info = NULL;
+	panel_applet_clean (info);
 }
 
 static void
@@ -72,35 +139,80 @@ static gchar popup_xml [] =
 void
 panel_applet_frame_load (const gchar *iid,
 			 PanelWidget *panel,
-			 gint         pos)
+			 gint         pos,
+			 const char  *gconf_key)
 {
-	GtkWidget  *frame;
+	GtkWidget  *frame = NULL;
 	AppletInfo *info;
+	char       *real_key;
 
-	frame = panel_applet_frame_new (iid);
+	g_return_if_fail (iid);
+	g_return_if_fail (panel);
 
+	if (gconf_key)
+		real_key = g_strdup (gconf_key);
+	else
+		real_key = gconf_unique_key ();
+
+	frame = panel_applet_frame_new (iid, real_key);
+
+	if (!frame) {
+		GtkWidget *dialog;
+
+		dialog = gtk_message_dialog_new (NULL,
+						 0,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("There was a problem loading the applet."));
+		
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		
+		return;
+	}
+	
 	gtk_widget_show_all (frame);
 
 	info = panel_applet_register (frame, 
-				      frame,     /* FIXME: ref? */
-				      NULL,      /* FIXME: data_destroy */
+				      NULL /* data */,
+				      NULL /* data_destroy */,
 				      panel,
 				      pos,
 				      FALSE,
-				      APPLET_BONOBO);
+				      APPLET_BONOBO,
+				      real_key);
 
 	if (!info)
 		g_warning (_("Cannot register control widget\n"));
 
+	g_free (real_key);
+
 	panel_applet_frame_set_info (PANEL_APPLET_FRAME (frame), info);
 }
 
 void
-panel_applet_frame_save_position (PanelAppletFrame *frame)
+panel_applet_frame_get_expand_flags (PanelAppletFrame *frame,
+				     gboolean         *expand_major,
+				     gboolean         *expand_minor)
 {
-	/*
-	 * FIXME: implement.
-	 */
+	CORBA_Environment  env;
+	CORBA_boolean major = 0, minor = 0;
+
+	CORBA_exception_init (&env);
+
+	GNOME_Vertigo_PanelAppletShell_getExpandFlags (frame->priv->applet_shell,
+						       &major, &minor,
+						       &env);
+	
+	if (BONOBO_EX (&env))
+		g_warning (G_STRLOC " : exception return from getExpandFlags '%s'",
+			   BONOBO_EX_REPOID (&env));
+	else {
+		*expand_major = major;
+		*expand_minor = minor;
+	}
+	
+	CORBA_exception_free (&env);
 }
 
 void
@@ -124,12 +236,23 @@ panel_applet_frame_change_size (PanelAppletFrame *frame,
 }
 
 void
-panel_applet_frame_change_background_pixmap (PanelAppletFrame *frame,
-					     gchar            *pixmap)
+panel_applet_frame_change_background_pixmap (PanelAppletFrame *frame)
 {
-	gchar *bg_str;
+	GdkNativeWindow  pixmap_xid;
+	PanelWidget     *panel_widget;
+	gchar           *bg_str;
 
-	bg_str = g_strdup_printf ("pixmap:%s", pixmap);
+	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
+	g_return_if_fail (PANEL_IS_WIDGET (GTK_WIDGET (frame)->parent));
+
+	panel_widget = PANEL_WIDGET (GTK_WIDGET (frame)->parent);
+
+	pixmap_xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (GTK_WIDGET (panel_widget)->window));
+
+	bg_str = g_strdup_printf ("pixmap:%d,%d,%d",
+				  pixmap_xid,
+				  GTK_WIDGET (frame)->allocation.x,
+				  GTK_WIDGET (frame)->allocation.y);
 
 	bonobo_pbclient_set_string (frame->priv->property_bag, 
 				    "panel-applet-background",
@@ -147,7 +270,7 @@ panel_applet_frame_change_background_color (PanelAppletFrame *frame,
 {
 	gchar *bg_str;
 
-	bg_str = g_strdup_printf ("color:#%.2x%.2x%.2x", red, green, blue);
+	bg_str = g_strdup_printf ("color:#%.4x%.4x%.4x", red, green, blue);
 
 	bonobo_pbclient_set_string (frame->priv->property_bag, 
 				    "panel-applet-background",
@@ -180,6 +303,9 @@ panel_applet_frame_finalize (GObject *object)
 {
 	PanelAppletFrame *frame = PANEL_APPLET_FRAME (object);
 
+	g_free (frame->priv->iid);
+	frame->priv->iid = NULL;
+
         g_free (frame->priv);
         frame->priv = NULL;
 
@@ -235,16 +361,16 @@ panel_applet_frame_get_type (void)
 	return type;
 }
 
-static GNOME_PanelAppletShell
+static GNOME_Vertigo_PanelAppletShell
 panel_applet_frame_get_applet_shell (Bonobo_Control control)
 {
-	CORBA_Environment      env;
-	GNOME_PanelAppletShell retval;
+	CORBA_Environment              env;
+	GNOME_Vertigo_PanelAppletShell retval;
 
 	CORBA_exception_init (&env);
 
 	retval = Bonobo_Unknown_queryInterface (control, 
-						"IDL:GNOME/PanelAppletShell:1.0",
+						"IDL:GNOME/Vertigo/PanelAppletShell:1.0",
 						&env);
 	if (BONOBO_EX (&env)) {
 		g_warning (_("Unable to obtain AppletShell interface from control\n"));
@@ -257,21 +383,33 @@ panel_applet_frame_get_applet_shell (Bonobo_Control control)
 	return retval;
 }
 
-void
-panel_applet_frame_construct (PanelAppletFrame  *frame,
-			      const gchar       *iid)
+GtkWidget *
+panel_applet_frame_construct (PanelAppletFrame *frame,
+			      const char       *iid,
+			      const char       *gconf_key)
 {
 	BonoboControlFrame *control_frame;
 	Bonobo_Control      control;
 	BonoboUIComponent  *ui_component;
 	GtkWidget          *widget;
+	gchar              *moniker;
+
+	moniker = g_strdup_printf ("%s!prefs_key=/apps/panel/profiles/%s/applets/%s/prefs", 
+				   iid,
+				   session_get_current_profile (),
+				   gconf_key);
+
+        widget = bonobo_widget_new_control (moniker, NULL);
+
+	g_free (moniker);
 
-        widget = bonobo_widget_new_control (iid, NULL);
 	if (!widget) {
 		g_warning (G_STRLOC ": failed to load %s", iid);
-		return;
+		return NULL;
 	}
 
+	frame->priv->iid = g_strdup (iid);
+
         control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
 
         control = bonobo_control_frame_get_control (control_frame);
@@ -301,16 +439,24 @@ panel_applet_frame_construct (PanelAppletFrame  *frame,
         bonobo_ui_component_add_verb_list_with_data (ui_component, popup_verbs, frame);
 
         gtk_container_add (GTK_CONTAINER (frame), widget);
+
+	return widget;
 }
 
 GtkWidget *
-panel_applet_frame_new (const gchar *iid)
+panel_applet_frame_new (const char *iid,
+			const char *gconf_key)
 {
 	PanelAppletFrame *frame;
 
+	g_return_val_if_fail (iid && gconf_key, NULL);
+
 	frame = g_object_new (PANEL_TYPE_APPLET_FRAME, NULL);
 
-	panel_applet_frame_construct (frame, iid);
+	if (!panel_applet_frame_construct (frame, iid, gconf_key)) {
+		g_object_unref (frame);
+		return NULL;
+	}
 
 	return GTK_WIDGET (frame);
 }
