@@ -16,10 +16,16 @@
 #include <fcntl.h>
 #include <dirent.h>
 
+/* includes for the battery stuff */
+#include <signal.h>
+#include <limits.h>
+#include <unistd.h>
+#include <sys/wait.h>
+
 #include <libgnomeui/libgnomeui.h>
 #include <libgnome/libgnome.h>
 
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -27,7 +33,8 @@
 #include "panel-util.h"
 
 #include "applet.h"
-#include "panel_config_global.h"
+#include "nothing.h"
+#include "panel-config-global.h"
 
 extern GlobalConfig global_config;
 
@@ -36,18 +43,23 @@ extern GSList *applets;
 void
 panel_show_help (const char *doc_name, const char *linkid)
 {
-	gnome_help_display (doc_name,
-			    linkid,
-			    NULL);
+	GError *error = NULL;
 
-	/* FIXME: handle error */
+	if ( ! gnome_help_display (doc_name,
+				   linkid,
+				   &error)) {
+		panel_error_dialog ("cannot_show_help",
+				    _("<b>Cannot display help document</b>\n\n"
+				      "Details: %s"), error->message);
+		g_clear_error (&error);
+	}
 }
 
-#ifdef FIXME
-static char *
-panel_gnome_help_path (const char *docpath)
+static gboolean
+panel_show_gnome_help (const char *docpath, GError **error)
 {
-	char *fullpath, *app, *p, *path, *uri;
+	char *app, *p, *path;
+	gboolean retval;
 
 	app = g_strdup (docpath);
 
@@ -55,36 +67,49 @@ panel_gnome_help_path (const char *docpath)
 
 	if (p == NULL) {
 		g_free (app);
-		return NULL;
+		g_set_error (error,
+			     PANEL_HELP_ERROR,
+			     PANEL_HELP_ERROR_NOT_FOUND,
+			     /* XXX: Not really a correct error,
+			      * but just somewhat correct */
+			     _("Help document not found"));
+		return FALSE;
 	}
 
 	path = p+1;
 	*p = '\0';
 
-	fullpath = gnome_help_file_path (app, path);
+	retval = TRUE;
+
+	if ( ! gnome_help_display_desktop (NULL /* program */,
+					   app /* doc_id */,
+					   path /* file_name */,
+					   NULL /* link_id */,
+					   error)) {
+		retval = gnome_help_display_with_doc_id (NULL /* program */,
+							 app /* doc_id */,
+							 path /* file_name */,
+							 NULL /* link_id */,
+							 error);
+	}
 
 	g_free (app);
 
-	if ( ! g_file_test (fullpath, G_FILE_TEST_EXISTS)) {
-		g_free (fullpath);
-		fullpath = NULL;
-	}
-
-	uri = g_strconcat ("ghelp:", fullpath, NULL);
-	g_free (fullpath);
-
-	return uri;
+	return retval;
 }
-#endif /* FIXME */
 
-#ifdef FIXME
-static char *
-panel_kde_help_path (const char *docpath)
+static gboolean
+panel_show_kde_help (const char *docpath, GError **error)
 {
 	const GList *li;
 
-	if ( ! g_file_test (KDE_DOCDIR, G_FILE_TEST_EXISTS))
-		return NULL;
+	if ( ! g_file_test (KDE_DOCDIR, G_FILE_TEST_EXISTS)) {
+		g_set_error (error,
+			     PANEL_HELP_ERROR,
+			     PANEL_HELP_ERROR_NOT_FOUND,
+			     _("Help document not found"));
+		return FALSE;
+	}
 
 	for (li = gnome_i18n_get_language_list ("LC_MESSAGES");
 	     li != NULL;
@@ -94,53 +119,60 @@ panel_kde_help_path (const char *docpath)
 						  (char *)li->data,
 						  docpath);
 		if (g_file_test (fullpath, G_FILE_TEST_EXISTS)) {
+			gboolean retval;
 			char *uri = g_strconcat ("ghelp:", fullpath, NULL);
 			g_free (fullpath);
-			return uri;
+			retval = gnome_help_display_uri (uri, error);
+			g_free (uri);
+			return retval;
 		}
 		g_free (fullpath);
 	}
 
-	return NULL;
-}
-#endif /* FIXME */
+	g_set_error (error,
+		     PANEL_HELP_ERROR,
+		     PANEL_HELP_ERROR_NOT_FOUND,
+		     _("Help document not found"));
 
-char *
-panel_gnome_kde_help_path (const char *docpath)
-{
-#ifdef FIXME
-	char *path;
-
-	if (string_empty (docpath))
-		return NULL;
-
-	if (panel_is_url (docpath))
-		return g_strdup (docpath);
-
-	path = panel_gnome_help_path (docpath);
-
-	if (path == NULL)
-		path = panel_kde_help_path (docpath);
-
-	return path;
-#else
-	return NULL;
-#endif
+	return FALSE;
 }
 
 gboolean
-string_is_in_list(const GSList *list, const char *text)
+panel_show_gnome_kde_help (const char *docpath,
+			   GError **error)
 {
-	for(;list != NULL; list = list->next)
-		if(strcmp(text,(char *)list->data) == 0)
-			return TRUE;
-	return FALSE;
+	if (string_empty (docpath)) {
+		g_set_error (error,
+			     PANEL_HELP_ERROR,
+			     PANEL_HELP_ERROR_NO_DOCPATH,
+			     _("No document to show"));
+		return FALSE;
+	}
+
+	if (panel_is_url (docpath))
+		return gnome_help_display_uri (docpath, error);
+
+	if ( ! panel_show_gnome_help (docpath, error)) {
+		return panel_show_kde_help (docpath, error);
+	}
+
+	return TRUE;
+}
+
+GQuark
+panel_help_error_quark (void)
+{
+	static GQuark q = 0;
+	if (q == 0)
+		q = g_quark_from_static_string ("panel-help-error-quark");
+
+	return q;
 }
 
 static void
 updated (GtkWidget *w, gpointer data)
 {
-	UpdateFunction func = gtk_object_get_data (GTK_OBJECT (w), "update_function");
+	UpdateFunction func = g_object_get_data (G_OBJECT (w), "update_function");
 
 	func (data);
 }
@@ -178,7 +210,7 @@ create_text_entry(GtkWidget *table,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	if(func) {
-		gtk_object_set_data (GTK_OBJECT (t), "update_function", func);
+		g_object_set_data (G_OBJECT (t), "update_function", func);
 		g_signal_connect (G_OBJECT (t), "changed",
 				  G_CALLBACK (updated), 
 				  data);
@@ -221,7 +253,7 @@ create_icon_entry(GtkWidget *table,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	if(func) {
-		gtk_object_set_data (GTK_OBJECT (entry), "update_function", func);
+		g_object_set_data (G_OBJECT (entry), "update_function", func);
 		g_signal_connect (G_OBJECT (entry), "changed",
 				  G_CALLBACK (updated), 
 				  data);
@@ -231,7 +263,7 @@ create_icon_entry(GtkWidget *table,
 }
 
 GList *
-my_g_list_swap_next (GList *list, GList *dl)
+panel_g_list_swap_next (GList *list, GList *dl)
 {
 	GList *t;
 
@@ -254,7 +286,7 @@ my_g_list_swap_next (GList *list, GList *dl)
 }
 
 GList *
-my_g_list_swap_prev (GList *list, GList *dl)
+panel_g_list_swap_prev (GList *list, GList *dl)
 {
 	GList *t;
 
@@ -279,7 +311,7 @@ my_g_list_swap_prev (GList *list, GList *dl)
 /*maybe this should be a glib function?
  it resorts a single item in the list*/
 GList *
-my_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
+panel_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
 {
 	GList *dl;
 
@@ -292,10 +324,10 @@ my_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
 
 	while(dl->next &&
 	      (*func)(dl->data,dl->next->data)>0)
-		list=my_g_list_swap_next(list,dl);
+		list = panel_g_list_swap_next (list, dl);
 	while(dl->prev &&
 	      (*func)(dl->data,dl->prev->data)<0)
-		list=my_g_list_swap_prev(list,dl);
+		list = panel_g_list_swap_prev (list, dl);
 	return list;
 }
 
@@ -493,7 +525,7 @@ set_color_back (GtkWidget *widget, PanelWidget *panel)
 	GtkStyle *ns;
 	int i;
 
-	gtk_widget_set_rc_style (widget);
+	gtk_widget_set_style (widget, NULL);
 	ns = gtk_style_copy (gtk_widget_get_style (widget));
 
 	ns->bg[GTK_STATE_NORMAL] =
@@ -514,13 +546,13 @@ set_color_back (GtkWidget *widget, PanelWidget *panel)
 		ns->mid[i].blue = (ns->light[i].blue + ns->dark[i].blue) / 2;
 	}
 	gtk_widget_set_style (widget, ns);
-	gtk_style_unref (ns);
+	g_object_unref (G_OBJECT (ns));
 }
 
 void
-set_frame_colors (PanelWidget *panel, GtkWidget *frame,
-		  GtkWidget *but1, GtkWidget *but2,
-		  GtkWidget *but3, GtkWidget *but4)
+panel_set_frame_colors (PanelWidget *panel, GtkWidget *frame,
+			GtkWidget *but1, GtkWidget *but2,
+			GtkWidget *but3, GtkWidget *but4)
 {
 	if (panel->back_type == PANEL_BACK_COLOR) {
 		set_color_back (frame, panel);
@@ -529,170 +561,31 @@ set_frame_colors (PanelWidget *panel, GtkWidget *frame,
 		set_color_back (but3, panel);
 		set_color_back (but4, panel);
 	} else {
-		gtk_widget_set_rc_style (frame);
-		gtk_widget_set_rc_style (but1);
-		gtk_widget_set_rc_style (but2);
-		gtk_widget_set_rc_style (but3);
-		gtk_widget_set_rc_style (but4);
+		gtk_widget_set_style (frame, NULL);
+		gtk_widget_set_style (but1, NULL);
+		gtk_widget_set_style (but2, NULL);
+		gtk_widget_set_style (but3, NULL);
+		gtk_widget_set_style (but4, NULL);
 	}
 }
 
-
-void
-remove_directory(const char *dirname, gboolean just_clean)
-{
-	DIR *dir;
-	struct dirent *dent;
-	char *oldcwd;
-
-	dir = opendir (dirname);
-	if(!dir) return;
-	oldcwd = g_get_current_dir();
-
-	chdir(dirname);
-	while((dent = readdir (dir)) != NULL) {
-		if(strcmp(dent->d_name,".")==0 ||
-		   strcmp(dent->d_name,"..")==0)
-			continue;
-		if(g_file_test(dent->d_name,G_FILE_TEST_IS_DIR))
-			remove_directory(dent->d_name, FALSE);
-		else
-			unlink(dent->d_name);
-	}
-	closedir(dir);
-	chdir(oldcwd);
-
-	if(!just_clean)
-		rmdir(dirname);
-	g_free(oldcwd);
-}
-
-char *
-strtok_with_escape(char *string, const char *seps, gboolean empty)
-{
-	char *our_string;
-	static char *p = NULL;
-
-	g_return_val_if_fail(seps != NULL, NULL);
-
-	if(string)
-		p = string;
-	else if(!p)
-		return NULL;
-
-	our_string = p;
-	while(*p) {
-		if(*p == '\\' && *(p+1)) {
-			strcpy(p,p+1);
-		} else if(strchr(seps,*p)) {
-			*p = '\0';
-			if(empty || *our_string) {
-				p++;
-				return our_string;
-			}
-
-			/* this was an empty part and
-			   we don't want to return empty
-			   parts, so skip it */
-			our_string = p+1;
-		}
-		p++;
-	}
-	p = NULL;
-
-	if(!empty && !*our_string)
-		return NULL;
-	return our_string;
-}
-
-/* return a newly allocated string that escapes / and 'special' */
-char *
-escape_string(const char *string, const char *special)
-{
-	int count;
-	const char *p;
-	char *ret;
-	int i;
-
-	if(!string) return NULL;
-
-	g_return_val_if_fail(special != NULL, NULL);
-
-	for(p=string,count=0; *p; p++,count++) {
-		if(*p == '\\' ||
-		   strchr(special,*p))
-			count++;
-	}
-
-	ret = g_new(char,count+1);
-	i = 0;
-	for(p=string; *p; p++) {
-		if(*p == '\\' ||
-		   strchr(special,*p))
-			ret[i++] = '\\';
-		ret[i++] = *p;
-	}
-	ret[i] = '\0';
-
-	return ret;
-}
-
 gboolean
 convert_string_to_keysym_state(const char *string,
 			       guint *keysym,
-			       guint *state)
+			       GdkModifierType *state)
 {
-	char *s, *p;
-
 	g_return_val_if_fail (keysym != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
 	
 	*state = 0;
 	*keysym = 0;
 
-	if(string_empty (string) ||
-	   strcmp (string, "Disabled") == 0 ||
-	   strcmp (string, _("Disabled")) == 0)
+	if (string_empty (string) ||
+	    strcmp (string, "Disabled") == 0 ||
+	    strcmp (string, _("Disabled")) == 0)
 		return FALSE;
 
-	s = g_strdup (string);
-
-	gdk_error_trap_push ();
-
-	p = strtok (s, "-");
-	while (p != NULL) {
-		if(strcmp(p, "Control")==0) {
-			*state |= GDK_CONTROL_MASK;
-		} else if(strcmp(p, "Lock")==0) {
-			*state |= GDK_LOCK_MASK;
-		} else if(strcmp(p, "Shift")==0) {
-			*state |= GDK_SHIFT_MASK;
-		} else if(strcmp(p, "Mod1")==0) {
-			*state |= GDK_MOD1_MASK;
-		} else if(strcmp(p, "Mod2")==0) {
-			*state |= GDK_MOD2_MASK;
-		} else if(strcmp(p, "Mod3")==0) {
-			*state |= GDK_MOD3_MASK;
-		} else if(strcmp(p, "Mod4")==0) {
-			*state |= GDK_MOD4_MASK;
-		} else if(strcmp(p, "Mod5")==0) {
-			*state |= GDK_MOD5_MASK;
-		} else {
-			*keysym = gdk_keyval_from_name(p);
-			if(*keysym == 0) {
-				gdk_flush();
-				gdk_error_trap_pop();
-				g_free(s);
-				return FALSE;
-			}
-		} 
-		p = strtok(NULL, "-");
-	}
-
-	gdk_flush ();
-	gdk_error_trap_pop ();
-
-	g_free (s);
+	gtk_accelerator_parse (string, keysym, state);
 
 	if (*keysym == 0)
 		return FALSE;
@@ -702,72 +595,12 @@ convert_string_to_keysym_state(const char *string,
 
 char *
 convert_keysym_state_to_string(guint keysym,
-			       guint state)
+			       GdkModifierType state)
 {
-	GString *gs;
-	char *sep = "";
-	char *key;
+	if (keysym == 0)
+		return g_strdup (_("Disabled"));
 
-	if(keysym == 0)
-		return g_strdup(_("Disabled"));
-
-	gdk_error_trap_push();
-	key = gdk_keyval_name(keysym);
-	gdk_flush();
-	gdk_error_trap_pop();
-	if(!key) return NULL;
-
-	gs = g_string_new(NULL);
-
-	if(state & GDK_CONTROL_MASK) {
-		/*g_string_append(gs, sep);*/
-		g_string_append(gs, "Control");
-		sep = "-";
-	}
-	if(state & GDK_LOCK_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Lock");
-		sep = "-";
-	}
-	if(state & GDK_SHIFT_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Shift");
-		sep = "-";
-	}
-	if(state & GDK_MOD1_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod1");
-		sep = "-";
-	}
-	if(state & GDK_MOD2_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod2");
-		sep = "-";
-	}
-	if(state & GDK_MOD3_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod3");
-		sep = "-";
-	}
-	if(state & GDK_MOD4_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod4");
-		sep = "-";
-	}
-	if(state & GDK_MOD5_MASK) {
-		g_string_append(gs, sep);
-		g_string_append(gs, "Mod5");
-		sep = "-";
-	}
-
-	g_string_append(gs, sep);
-	g_string_append(gs, key);
-
-	{
-		char *ret = gs->str;
-		g_string_free(gs, FALSE);
-		return ret;
-	}
+	return gtk_accelerator_name (keysym, state);
 }
 
 GtkWidget *
@@ -789,9 +622,15 @@ panel_error_dialog (const char *class,
 	}
 
 	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
-				    GTK_BUTTONS_OK, s);
+				    GTK_BUTTONS_OK, "foo");
+	gtk_widget_add_events (w, GDK_KEY_PRESS_MASK);
+	g_signal_connect (G_OBJECT (w), "event",
+			  G_CALLBACK (panel_dialog_window_event), NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (w),
 				class, "Panel");
+
+	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), s);
+
 	g_free (s);
 
 	gtk_widget_show_all (w);
@@ -822,9 +661,14 @@ panel_info_dialog (const char *class,
 	}
 
 	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
-				    GTK_BUTTONS_OK, s);
+				    GTK_BUTTONS_OK, "foo");
+	g_signal_connect (G_OBJECT (w), "event",
+			  G_CALLBACK (panel_dialog_window_event), NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (w),
 				class, "Panel");
+
+	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), s);
+
 	g_free (s);
 
 	gtk_widget_show_all (w);
@@ -836,16 +680,6 @@ panel_info_dialog (const char *class,
 	return w;
 }
 
-gboolean
-string_empty (const char *string)
-{
-	if (string == NULL ||
-	    string[0] == '\0')
-		return TRUE;
-	else
-		return FALSE;
-}
-
 gboolean
 is_ext (const char *file, const char *ext)
 {
@@ -884,7 +718,7 @@ is_ext2 (const char *file,
 }
 
 int
-find_applet (GtkWidget *widget)
+panel_find_applet (GtkWidget *widget)
 {
 	int i;
 	GSList *li;
@@ -937,57 +771,11 @@ panel_is_url (const char *url)
 		return FALSE;
 }
 
-static gboolean
-no_need_to_quote (const char *str)
-{
-	const char *p;
-	
-	/* empty string needs quoting */
-	if (string_empty (str))
-		return TRUE;
-
-	for (p = str; *p != '\0'; p++) {
-		if ((*p >= 'a' && *p <= 'z') ||
-		    (*p >= 'A' && *p <= 'Z') ||
-		    strchr ("_-./", *p) != NULL)
-			continue;
-
-		return FALSE;
-	}
-	return TRUE;
-}
-
-/* Quote string so that it could be on a shell like
- * quoted line as an argument */
-char *
-panel_quote_string (const char *str)
-{
-	GString *gs;
-	const char *p;
-
-	if (string_empty (str))
-		return g_strdup ("''");
-
-	if (no_need_to_quote (str))
-		return g_strdup (str);
-
-	gs = g_string_new ("'");
-	for (p = str; *p != '\0'; p++) {
-		if (*p == '\'')
-			g_string_append (gs, "'\\''");
-		else
-			g_string_append_c (gs, *p);
-	}
-	g_string_append_c (gs, '\'');
-
-	return g_string_free (gs, FALSE);
-}
-
 void
 panel_push_window_busy (GtkWidget *window)
 {
-	int busy = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (window),
-							 "Panel:WindowBusy"));
+	int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window),
+						       "Panel:WindowBusy"));
 
 	busy ++;
 
@@ -996,31 +784,31 @@ panel_push_window_busy (GtkWidget *window)
 		if (window->window != NULL) {
 			GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
 			gdk_window_set_cursor (window->window, cursor);
-			gdk_cursor_destroy (cursor);
+			gdk_cursor_unref (cursor);
 			gdk_flush ();
 		}
 	}
 
-	gtk_object_set_data (GTK_OBJECT (window), "Panel:WindowBusy",
-			     GINT_TO_POINTER (busy));
+	g_object_set_data (G_OBJECT (window), "Panel:WindowBusy",
+			   GINT_TO_POINTER (busy));
 }
 
 void
 panel_pop_window_busy (GtkWidget *window)
 {
-	int busy = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (window),
-							 "Panel:WindowBusy"));
+	int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window),
+						       "Panel:WindowBusy"));
 	busy --;
 
 	if (busy <= 0) {
 		gtk_widget_set_sensitive (window, TRUE);
 		if (window->window != NULL)
 			gdk_window_set_cursor (window->window, NULL);
-		gtk_object_remove_data (GTK_OBJECT (window),
-					"Panel:WindowBusy");
+		g_object_set_data (G_OBJECT (window),
+				   "Panel:WindowBusy", NULL);
 	} else {
-		gtk_object_set_data (GTK_OBJECT (window), "Panel:WindowBusy",
-				     GINT_TO_POINTER (busy));
+		g_object_set_data (G_OBJECT (window), "Panel:WindowBusy",
+				   GINT_TO_POINTER (busy));
 	}
 }
 
@@ -1187,7 +975,7 @@ typedef struct {
 } StrechSides;
 
 static void
-strech_widget_realize (GtkWidget *widget)
+stretch_widget_realize (GtkWidget *widget)
 {
 	GdkWindowAttr attributes = {0};
 	int attributes_mask;
@@ -1256,7 +1044,7 @@ strech_widget_realize (GtkWidget *widget)
 }
 
 static void
-strech_widget_unrealize (GtkWidget *widget)
+stretch_widget_unrealize (GtkWidget *widget)
 {
 	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
 						 "StrechEventWindow");
@@ -1289,7 +1077,7 @@ raise_in_idle (gpointer data)
 
 
 static void
-strech_widget_map (GtkWidget *widget)
+stretch_widget_map (GtkWidget *widget)
 {
 	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
 						 "StrechEventWindow");
@@ -1305,7 +1093,7 @@ strech_widget_map (GtkWidget *widget)
 }
 
 static void
-strech_widget_unmap (GtkWidget *widget)
+stretch_widget_unmap (GtkWidget *widget)
 {
 	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
 						 "StrechEventWindow");
@@ -1317,7 +1105,7 @@ strech_widget_unmap (GtkWidget *widget)
 
 
 static void
-strech_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+stretch_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
 						 "StrechEventWindow");
@@ -1325,22 +1113,22 @@ strech_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		return;
 
 	/* somewhat evil */
-	strech_widget_unrealize (widget);
-	strech_widget_realize (widget);
-	strech_widget_map (widget);
+	stretch_widget_unrealize (widget);
+	stretch_widget_realize (widget);
+	stretch_widget_map (widget);
 }
 
 static void
-strech_widget_hierarchy_changed (GtkWidget *widget,
+stretch_widget_hierarchy_changed (GtkWidget *widget,
 				 GtkWidget *previous_toplevel)
 {
-	strech_widget_unrealize (widget);
-	strech_widget_realize (widget);
-	strech_widget_map (widget);
+	stretch_widget_unrealize (widget);
+	stretch_widget_realize (widget);
+	stretch_widget_map (widget);
 }
 
 void
-panel_strech_events_to_toplevel (GtkWidget *widget,
+panel_stretch_events_to_toplevel (GtkWidget *widget,
 				 gboolean top,
 				 gboolean right,
 				 gboolean bottom,
@@ -1349,22 +1137,22 @@ panel_strech_events_to_toplevel (GtkWidget *widget,
 	StrechSides *sides;
 
 	g_signal_connect_after (GTK_WIDGET (widget), "realize",
-				G_CALLBACK (strech_widget_realize),
+				G_CALLBACK (stretch_widget_realize),
 				NULL);
 	g_signal_connect (GTK_WIDGET (widget), "unrealize",
-			  G_CALLBACK (strech_widget_unrealize),
+			  G_CALLBACK (stretch_widget_unrealize),
 			  NULL);
 	g_signal_connect_after (GTK_WIDGET (widget), "size_allocate",
-				G_CALLBACK (strech_widget_size_allocate),
+				G_CALLBACK (stretch_widget_size_allocate),
 				NULL);
 	g_signal_connect_after (GTK_WIDGET (widget), "map",
-				G_CALLBACK (strech_widget_map),
+				G_CALLBACK (stretch_widget_map),
 				NULL);
 	g_signal_connect_after (GTK_WIDGET (widget), "unmap",
-				G_CALLBACK (strech_widget_unmap),
+				G_CALLBACK (stretch_widget_unmap),
 				NULL);
 	g_signal_connect_after (GTK_WIDGET (widget), "hierarchy_changed",
-				G_CALLBACK (strech_widget_hierarchy_changed),
+				G_CALLBACK (stretch_widget_hierarchy_changed),
 				NULL);
 
 	sides = g_new0 (StrechSides, 1);
@@ -1378,3 +1166,178 @@ panel_strech_events_to_toplevel (GtkWidget *widget,
 				sides,
 				(GDestroyNotify) g_free);
 }
+
+/* stolen from gtk */
+void
+panel_signal_connect_while_alive (GObject     *object,
+				  const gchar *signal,
+				  GCallback    func,
+				  gpointer     func_data,
+				  GObject     *alive_object)
+{
+	GClosure *closure;
+
+	g_return_if_fail (G_IS_OBJECT (object));
+
+	closure = g_cclosure_new (func, func_data, NULL);
+	g_object_watch_closure (G_OBJECT (alive_object), closure);
+	g_signal_connect_closure_by_id (object,
+					g_signal_lookup (signal, G_OBJECT_TYPE (object)), 0,
+					closure,
+					FALSE);
+}
+
+void
+panel_signal_connect_object_while_alive (GObject      *object,
+					 const gchar  *signal,
+					 GCallback     func,
+					 GObject      *alive_object)
+{
+  g_return_if_fail (G_IS_OBJECT (object));
+  
+  g_signal_connect_closure_by_id (object,
+				  g_signal_lookup (signal, G_OBJECT_TYPE (object)), 0,
+				  g_cclosure_new_object_swap (func, G_OBJECT (alive_object)),
+				  FALSE);
+}
+
+gboolean
+panel_ensure_dir (const char *dirname)
+{
+	char *parsed, *p;
+
+	if (dirname == NULL)
+		return FALSE;
+
+	parsed = g_strdup (dirname);
+
+	if (g_file_test (parsed, G_FILE_TEST_IS_DIR)) {
+		g_free (parsed);
+		return TRUE;
+	}
+
+	p = strchr (parsed, '/');
+	if (p == parsed)
+		p = strchr (p+1, '/');
+
+	while (p != NULL) {
+		*p = '\0';
+		if (mkdir (parsed, 0700) != 0 &&
+		    errno != EEXIST) {
+			g_free (parsed);
+			return FALSE;
+		}
+		*p = '/';
+		p = strchr (p+1, '/');
+	}
+
+	if (mkdir (parsed, 0700) != 0 &&
+	    errno != EEXIST) {
+		g_free (parsed);
+		return FALSE;
+	}
+
+	g_free (parsed);
+	return TRUE;
+}
+
+void
+panel_g_list_deep_free	(GList *list)
+{
+	GList *li;
+	for (li = list; li != NULL; li = li->next) {
+		g_free (li->data);
+		li->data = NULL;
+	}
+	g_list_free (list);
+}
+
+void
+panel_g_slist_deep_free	(GSList *list)
+{
+	GSList *li;
+	for (li = list; li != NULL; li = li->next) {
+		g_free (li->data);
+		li->data = NULL;
+	}
+	g_slist_free (list);
+}
+
+#if 0
+/* hmmm, we need to use this I think to add a battery applet
+ *  -George */
+/* Linux Battery Stuff */
+
+static gboolean
+linux_battery_exists (void)
+{
+        FILE *fp;
+        char buf[200] = "";
+        int foo;
+
+        if ( ! g_file_test ("/proc/apm", G_FILE_TEST_EXISTS))
+                return FALSE;
+
+        fp = fopen ("/proc/apm", "r");
+        if (fp == NULL)
+                return FALSE;
+
+        if (fgets (buf, sizeof (buf), fp) == NULL) {
+                fclose (fp);
+                return FALSE;
+        }
+        fclose (fp);
+
+        foo = -1;
+        sscanf (buf,
+                "%*s %*d.%*d %*x %*x %*x %*x %d%% %*d %*s\n",
+                &foo);
+
+        if (foo >= 0)
+                return TRUE;
+        else
+                return FALSE;
+}
+
+static gboolean
+battery_exists (void)
+{
+#ifndef __linux__
+        return FALSE;
+#else
+        /* This is MUUUUCHO ugly, but apparently RH 7.0 with segv the program
+         * reading /proc/apm on some laptops, and we don't want to crash, thus
+         * we do the check in a forked process */
+        int status;
+        pid_t pid;
+
+        pid = fork ();
+        if (pid == 0) {
+                struct sigaction sa = {{NULL}};
+
+                sa.sa_handler = SIG_DFL;
+
+                sigaction(SIGSEGV, &sa, NULL);
+                sigaction(SIGFPE, &sa, NULL);
+                sigaction(SIGBUS, &sa, NULL);
+
+                if (linux_battery_exists ())
+                        _exit (0);
+                else
+                        _exit (1);
+        }
+
+        status = 0;
+        waitpid (pid, &status, 0);
+
+        if ( ! WIFSIGNALED (status) &&
+            WIFEXITED (status) &&
+            WEXITSTATUS (status) == 0) {
+                return TRUE;
+        } else {
+                return FALSE;
+        }
+#endif
+}
+
+#endif
