@@ -10,12 +10,10 @@ G_BEGIN_DECLS
 
 typedef void (*UpdateFunction) (gpointer);
 
-gboolean	string_is_in_list	(const GSList *list,
-					 const char *text);
+/* TRUE if string is NULL or the first character is '\0' */
+#define		string_empty(s)		((s)==NULL||((char *)(s))[0]=='\0')
 
-/* return TRUE if string is NULL or the first character is '\0' */
-gboolean	string_empty		(const char *string);
-#define		sure_string(s)		((s)!=NULL?(s):"")
+#define		sure_string(s)		((const char *)((s)!=NULL?(s):""))
 
 GtkWidget *	create_text_entry	(GtkWidget *table,
 					 const char *history_id,
@@ -37,44 +35,31 @@ GtkWidget *	create_icon_entry	(GtkWidget *table,
 void		panel_show_help		(const char *path,
 					 const char *linkid);
 
-GList *		my_g_list_swap_next	(GList *list,
+GList *		panel_g_list_swap_next	(GList *list,
 					 GList *dl);
-GList *		my_g_list_swap_prev	(GList *list,
+GList *		panel_g_list_swap_prev	(GList *list,
 					 GList *dl);
 /*maybe this should be a glib function?
  it resorts a single item in the list*/
-GList *		my_g_list_resort_item	(GList *list,
+GList *		panel_g_list_resort_item(GList *list,
 					 gpointer data,
 					 GCompareFunc func);
 
-void		set_frame_colors	(PanelWidget *panel,
+void		panel_g_list_deep_free	(GList *list);
+void		panel_g_slist_deep_free	(GSList *list);
+
+void		panel_set_frame_colors	(PanelWidget *panel,
 					 GtkWidget *frame,
 					 GtkWidget *but1,
 					 GtkWidget *but2,
 					 GtkWidget *but3,
 					 GtkWidget *but4);
 
-/* recursively remove directory, if just_clean is true, then
-   the directory itself is also removed, otherwise just it's
-   contents, it does not give any error output and will fail
-   silently, but it will remove as much as possible */
-void		remove_directory	(const char *dirname,
-					 gboolean just_clean);
-
-/* like strtok, but unescapes string as well and escaped separators
-   don't count.  if empty is true, don't skip empty parts */
-char *		strtok_with_escape	(char *string,
-					 const char *seps,
-					 gboolean empty);
-/* return a newly allocated string that escapes / and 'special' */
-char *		escape_string		(const char *string,
-					 const char *special);
-
 gboolean	convert_string_to_keysym_state (const char *string,
 						guint *keysym,
-						guint *state);
+						GdkModifierType *state);
 char *		convert_keysym_state_to_string (guint keysym,
-						guint state);
+						GdkModifierType state);
 
 GtkWidget *	panel_error_dialog	(const char *class,
 					 const char *format,
@@ -89,19 +74,24 @@ gboolean	is_ext2			(const char *file,
 					 const char *ext1,
 					 const char *ext2);
 
-int		find_applet		(GtkWidget *widget);
+int		panel_find_applet	(GtkWidget *widget);
 
 int		get_requisition_width	(GtkWidget *widget);
 int		get_requisition_height	(GtkWidget *widget);
 
-char *		panel_gnome_kde_help_path (const char *docpath);
+typedef enum {
+	PANEL_HELP_ERROR_NO_DOCPATH /* No docpath sent */,
+	PANEL_HELP_ERROR_NOT_FOUND /* Document not found */
+} PanelHelpError;
+
+#define PANEL_HELP_ERROR panel_help_error_quark ()
+GQuark panel_help_error_quark (void);
+
+gboolean	panel_show_gnome_kde_help (const char *docpath,
+					   GError **error);
 
 gboolean	panel_is_url		(const char *url);
 
-/* Quote string so that it could be on a shell like
- * quoted line as an argument */
-char *		panel_quote_string	(const char *str);
-
 void		panel_push_window_busy	(GtkWidget *window);
 void		panel_pop_window_busy	(GtkWidget *window);
 
@@ -110,13 +100,24 @@ gboolean	panel_is_program_in_path (const char *program);
 char *		panel_pixmap_discovery	(const char *name,
 					 gboolean fallback);
 
-void		panel_strech_events_to_toplevel (GtkWidget *widget,
+void		panel_stretch_events_to_toplevel (GtkWidget *widget,
 						 gboolean top,
 						 gboolean right,
 						 gboolean bottom,
 						 gboolean left);
 
 
+void		panel_signal_connect_while_alive (GObject     *object,
+						  const gchar *signal,
+						  GCallback    func,
+						  gpointer     func_data,
+						  GObject     *alive_object);
+void		panel_signal_connect_object_while_alive (GObject      *object,
+							 const gchar  *signal,
+							 GCallback     func,
+							 GObject      *alive_object);
+
+gboolean	panel_ensure_dir	(const char *dirname);
 
 /* GnomeVFS reading utils */
 
@@ -132,6 +133,10 @@ ReadBuf *	readbuf_open		(const char *uri);
 gboolean	readbuf_rewind		(ReadBuf *rb);
 void		readbuf_close		(ReadBuf *rb);
 
+
+/* FIXME : make battery stuff public as appropriate */
+
+
 G_END_DECLS
 
 #endif /* PANEL_UTIL_H */
