@@ -124,6 +124,8 @@ struct _PanelWidgetClass
 			      GdkColor *color);
 	void (* applet_about_to_die) (PanelWidget *panel,
 				      GtkWidget *applet);
+	void (* end_move) (PanelWidget	*panel);
+
 };
 
 GType		panel_widget_get_type		(void) G_GNUC_CONST;
@@ -223,6 +225,18 @@ int		panel_widget_get_cursorloc	(PanelWidget *panel);
 gboolean	panel_widget_is_cursor		(PanelWidget *panel,
 						 int overlap);
 
+
+void            panel_widget_push_applet        (PanelWidget *panel,
+                                                 AppletData  *ad,
+                                                 GtkDirectionType dir);
+
+void            panel_widget_switch_applet      (PanelWidget *panel,
+                                                 AppletData  *ad,
+                                                 GtkDirectionType dir);
+
+void            panel_widget_free_move_applet   (PanelWidget *panel,
+                                                 AppletData  *ad,
+                                                 GtkDirectionType dir);
 extern gboolean panel_applet_in_drag;
 
 G_END_DECLS
